create type statement;

create function statement_in(cstring) returns statement
as
'MODULE_PATHNAME',
'statement_in'
    language c strict
               immutable;

create function statement_out(statement) returns cstring
as
'MODULE_PATHNAME',
'statement_out'
    language c strict
               immutable;

create type statement
(
    input = statement_in,
    output = statement_out,
    like = text
);

create function is_parameterized(statement) returns bool
as
'MODULE_PATHNAME',
'is_parameterized'
    language c strict
               immutable;

create function is_valid(statement) returns bool
as
'MODULE_PATHNAME',
'is_valid'
    language c strict
               immutable;
;
create function add_cte(statement, name text, cte statement,
                        recursive bool default false, prepend bool default false) returns statement
as
'MODULE_PATHNAME',
'add_cte'
    language c strict
               immutable;

;
create type raw_statement as
(
    source text,
    line   int,
    col    int
);

create function raw_statements(cstring, boolean default false) returns setof raw_statement
as
'MODULE_PATHNAME'
    language c strict
               immutable;
;
create type statement_type as enum (
    'MultiStmt', -- special type for multiple statements
    'InsertStmt', 'DeleteStmt', 'UpdateStmt', 'MergeStmt', 'SelectStmt', 'SetOperationStmt', 'ReturnStmt',
    'PLAssignStmt', 'CreateSchemaStmt', 'AlterTableStmt', 'ReplicaIdentityStmt', 'AlterCollationStmt', 'AlterDomainStmt',
    'GrantStmt', 'GrantRoleStmt', 'AlterDefaultPrivilegesStmt', 'CopyStmt', 'VariableSetStmt', 'VariableShowStmt',
    'CreateStmt', 'CreateTableSpaceStmt', 'DropTableSpaceStmt', 'AlterTableSpaceOptionsStmt', 'AlterTableMoveAllStmt',
    'CreateExtensionStmt', 'AlterExtensionStmt', 'AlterExtensionContentsStmt', 'CreateFdwStmt', 'AlterFdwStmt',
    'CreateForeignServerStmt', 'AlterForeignServerStmt', 'CreateForeignTableStmt', 'CreateUserMappingStmt',
    'AlterUserMappingStmt', 'DropUserMappingStmt', 'ImportForeignSchemaStmt', 'CreatePolicyStmt', 'AlterPolicyStmt',
    'CreateAmStmt', 'CreateTrigStmt', 'CreateEventTrigStmt', 'AlterEventTrigStmt', 'CreatePLangStmt', 'CreateRoleStmt',
    'AlterRoleStmt', 'AlterRoleSetStmt', 'DropRoleStmt', 'CreateSeqStmt', 'AlterSeqStmt', 'DefineStmt', 'CreateDomainStmt',
    'CreateOpClassStmt', 'CreateOpFamilyStmt', 'AlterOpFamilyStmt', 'DropStmt', 'TruncateStmt', 'CommentStmt', 'SecLabelStmt',
    'DeclareCursorStmt', 'ClosePortalStmt', 'FetchStmt', 'IndexStmt', 'CreateStatsStmt', 'AlterStatsStmt', 'CreateFunctionStmt',
    'AlterFunctionStmt', 'DoStmt', 'CallStmt', 'RenameStmt', 'AlterObjectDependsStmt', 'AlterObjectSchemaStmt',
    'AlterOwnerStmt', 'AlterOperatorStmt', 'AlterTypeStmt', 'RuleStmt', 'NotifyStmt', 'ListenStmt', 'UnlistenStmt',
    'TransactionStmt', 'CompositeTypeStmt', 'CreateEnumStmt', 'CreateRangeStmt', 'AlterEnumStmt', 'ViewStmt', 'LoadStmt',
    'CreatedbStmt', 'AlterDatabaseStmt', 'AlterDatabaseRefreshCollStmt', 'AlterDatabaseSetStmt', 'DropdbStmt',
    'AlterSystemStmt', 'ClusterStmt', 'VacuumStmt', 'ExplainStmt', 'CreateTableAsStmt', 'RefreshMatViewStmt',
    'CheckPointStmt', 'DiscardStmt', 'LockStmt', 'ConstraintsSetStmt', 'ReindexStmt', 'CreateConversionStmt', 'CreateCastStmt',
    'CreateTransformStmt', 'PrepareStmt', 'ExecuteStmt', 'DeallocateStmt', 'DropOwnedStmt', 'ReassignOwnedStmt',
    'AlterTSDictionaryStmt', 'AlterTSConfigurationStmt', 'CreatePublicationStmt', 'AlterPublicationStmt',
    'CreateSubscriptionStmt', 'AlterSubscriptionStmt', 'DropSubscriptionStmt', 'PlannedStmt'
    );

create function statement_type(statement) returns statement_type
    language sql
    strict immutable as
$$
select omni_sql._statement_type($1)::text::omni_sql.statement_type
$$;

create function _statement_type(statement) returns cstring
as
'MODULE_PATHNAME',
'statement_type'
    language c strict
               immutable;;
create function is_returning_statement(statement) returns boolean
as
'MODULE_PATHNAME' language c strict
  immutable;


comment on function omni_sql.is_returning_statement is 'Does the statement returns rows? In other words, can a cursor(portal more precisely) be created?';
;
create function is_replace_statement(statement) returns boolean
as
'MODULE_PATHNAME' language c strict
  immutable;


 comment on function omni_sql.is_replace_statement is 'Does the statement contain a OR REPLACE clause?';
;
create function execute_parameterized(stmt text, parameters jsonb, types regtype[]) returns setof record
    language c as
'MODULE_PATHNAME';

create function execute(stmt text, parameters jsonb default NULL,
                        types regtype[] default NULL::regtype[])
    returns table
            (
                stmt_row jsonb
            )
as
$pgsql$
declare
    rec            record;
    retrec         record;
begin
    for rec in select source                                              as stmt,
                      coalesce(not lead(true) over (), true)              as last
               from omni_sql.raw_statements(stmt::cstring)
        loop
            if not rec.last then
                execute rec.stmt;
            else
                for retrec in select omni_sql.execute_parameterized(rec.stmt, parameters, types) val
                    loop
                        stmt_row := to_jsonb(retrec.val);
                        return next;
                    end loop;
            end if;
        end loop;
    return;
end;
$pgsql$
    language plpgsql;


comment on function omni_sql.execute is 'Runs an arbitrary SQL statement. If the statement is a parameterized statement, one can define more precise types for the statement using the "types" parameter.'
;
