create domain params as text[] check (array_length(value, 1) % 2 = 0);

create function parse_query_string(query_string text) returns params
    strict immutable
as
'MODULE_PATHNAME',
'parse_query_string' language c;

create function parse_query_string(query_string bytea) returns params
    strict immutable
as
$$
select omni_web.parse_query_string(convert_from(query_string, 'UTF8'));
$$ language sql;;
create function param_get_all(params params, param text) returns setof text
    strict immutable
as
$$
select
    params[i + 1] as value
from
    unnest(params) with ordinality t(v, i)
where
    v = param and
    i % 2 = 1
order by
    i asc;
$$
    language sql;

create function param_get_all(params text, param text) returns setof text
    strict immutable
as
$$
select omni_web.param_get_all(omni_web.parse_query_string(params), param);
$$
    language sql;

create function param_get_all(params bytea, param text) returns setof text
    strict immutable
as
$$
select omni_web.param_get_all(omni_web.parse_query_string(convert_from(params, 'UTF8')), param);
$$
    language sql;

create function param_get(params params, param text) returns text
    strict immutable
as
$$
select
    param_get_all
from
    omni_web.param_get_all(params, param)
limit 1
$$
    language sql;

create function param_get(params text, param text) returns text
    strict immutable
as
$$
select omni_web.param_get(omni_web.parse_query_string(params), param);
$$
    language sql;

create function param_get(params bytea, param text) returns text
    strict immutable
as
$$
select omni_web.param_get(omni_web.parse_query_string(convert_from(params, 'UTF8')), param);
$$
    language sql;
;
create function cookies(cookies text)
    returns table
            (
                name  text,
                value text
            )
    immutable
    language plpgsql
as
$$
declare
    cookie text;
begin
    foreach cookie in array string_to_array(cookies, '; ')
        loop
            name := split_part(cookie, '=', 1);
            value := split_part(cookie, '=', 2);
            return next;
        end loop;
    return;
end;
$$;;
create function url_encode(text)
    returns text
as
'MODULE_PATHNAME'
    language c
    immutable
    strict;
create function url_decode(text)
    returns text
as
'MODULE_PATHNAME'
    language c
    immutable
    strict;

create function uri_encode(text)
    returns text
as
'MODULE_PATHNAME'
    language c
    immutable
    strict;
create function uri_decode(text)
    returns text
as
'MODULE_PATHNAME'
    language c
    immutable
    strict;;
create type uri as
(
    scheme    text,
    user_info text,
    host      text,
    path      text,
    port integer,
    query     text,
    fragment  text
);

create function text_to_uri(text) returns uri
    immutable
    strict
    language c as
'MODULE_PATHNAME';

create cast (text as uri) with function text_to_uri(text);;
create or replace function cookie_get(cookies text, cookie_name text)
    returns text
    strict
    immutable
    language plpgsql
as
$$
declare
    cookie text;
begin
    foreach cookie in array string_to_array
                            (cookies, '; '
                            )
        loop
            if split_part(cookie, '=', 1) = cookie_name then
                return split_part(cookie, '=', 2);
            end if;
        end loop;
    return null;
end;
$$;;
create type urlpattern;

create function urlpattern_in(cstring) returns urlpattern
    immutable
    language c as
'MODULE_PATHNAME';

create function urlpattern_out(urlpattern)
    returns cstring
    immutable
    language c as
'MODULE_PATHNAME';

create type urlpattern
(
    like = text,
    input = urlpattern_in,
    output = urlpattern_out
);

create function matches(urlpattern, text, baseURL text default null)
    returns bool
    immutable
    language c as
'MODULE_PATHNAME',
'urlpattern_matches';

create function match(urlpattern, text, baseURL text default null)
    returns table
            (
                name      text,
                component text,
                value     text
            )
    immutable
    language c
as
'MODULE_PATHNAME',
'urlpattern_match';;
