create type mime_type_source as enum ('iana', 'apache', 'nginx');

create table mime_types
(
    id           integer primary key generated always as identity,
    name         text not null unique,
    source       mime_type_source,
    compressible bool,
    charset      text
);

create table file_extensions
(
    id        integer primary key generated always as identity,
    extension text not null unique
);

create table mime_types_file_extensions
(
    mime_type_id      integer references mime_types (id),
    file_extension_id integer references file_extensions (id)
);


select pg_catalog.pg_extension_config_dump('mime_types', '');
select pg_catalog.pg_extension_config_dump('mime_types_id_seq', '');
select pg_catalog.pg_extension_config_dump('file_extensions', '');
select pg_catalog.pg_extension_config_dump('file_extensions_id_seq', '');
select pg_catalog.pg_extension_config_dump('mime_types_file_extensions', '');;
create function import_mime_db(data jsonb) returns void
as
$$
declare
    item          record;
    mime_type_id_ integer;
begin
    for item in select (jsonb_each(data)).*
        loop
            -- Insert mime type
            insert
            into
                omni_mimetypes.mime_types (name, source, compressible, charset)
                (select
                     item.key,
                     (item.value ->> 'source')::omni_mimetypes.mime_type_source,
                     (item.value ->> 'compressible')::bool,
                     item.value ->> 'charset')
            on conflict (name) do update set
                                             source       = (item.value ->> 'source')::omni_mimetypes.mime_type_source,
                                             compressible = (item.value ->> 'compressible')::bool,
                                             charset      = item.value ->> 'compressible'
            returning id into mime_type_id_;
            -- Insert extensions
            if item.value ? 'extensions' then
                -- Account for potential disassociation of extensions
                delete from omni_mimetypes.mime_types_file_extensions where mime_type_id = mime_type_id_;
                -- Insert extensions and their associations
                raise notice '%s', (item.value ->> 'extensions');
                with
                    extensions as
                        (insert into omni_mimetypes.file_extensions (extension) (select jsonb_array_elements_text(item.value -> 'extensions'))
                            on conflict (extension) do update set extension = file_extensions.extension returning id)
                insert
                into
                    omni_mimetypes.mime_types_file_extensions (mime_type_id, file_extension_id)
                select
                    mime_type_id_,
                    id
                from
                    extensions;
            end if;
        end loop;
    return;
end;
$$ language plpgsql;

create function import_mime_db(data json) returns void
as
$$
select omni_mimetypes.import_mime_db(data::jsonb)
$$ language sql;;
-- The following dump is generated with this:

/*
create extension omni_httpc cascade;
with
    db as (select *
           from
               omni_httpc.http_execute(omni_httpc.http_request('https://cdn.jsdelivr.net/gh/jshttp/mime-db@1.52.0/db.json')) as import)
select
    omni_mimetypes.import_mime_db(convert_from(body, 'utf8')::jsonb)
from
    db;
 */

-- And collected with:
-- pg_dump -Fp --inserts -h localhost -p PORT -a --no-owner omni_mimetypes

--
-- Data for Name: file_extensions; Type: TABLE DATA; Schema: omni_mimetypes; Owner: -
--

INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1, 'n3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (2, 'xm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (3, 'otf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (4, 'ttf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (5, 'css');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (6, 'csv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (7, 'jsx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (8, 'mdx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (9, 'rtf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (10, 'vtt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (11, 'c');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (12, 'cc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (13, 'cxx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (14, 'cpp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (15, 'h');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (16, 'hh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (17, 'dic');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (18, 'xml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (19, 'amr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (20, 'mp3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (21, 'm4a');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (22, 'mp4a');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (23, 'oga');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (24, 'ogg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (25, 'spx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (26, 'opus');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (27, 's3m');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (28, 'wav');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (29, 'woff');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (30, 'bmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (31, 'cgm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (32, 'emf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (33, 'gif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (34, 'ief');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (35, 'jls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (36, 'jp2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (37, 'jpg2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (38, 'jph');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (39, 'jpm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (40, 'jpx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (41, 'jpf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (42, 'jxr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (43, 'jxs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (44, 'ktx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (45, 'png');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (46, 'sgi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (47, 't38');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (48, 'wmf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (49, '3mf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (50, 'mtl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (51, 'obj');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (52, 'stl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (53, 'html');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (54, 'htm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (55, 'shtml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (56, 'jade');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (57, 'less');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (58, 'sgml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (59, 'sgm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (60, 'shex');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (61, 'slim');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (62, 'slm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (63, 'spdx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (64, 'yaml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (65, 'yml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (67, 'jpgm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (68, 'mj2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (69, 'mjp2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (70, 'mp4');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (71, 'mp4v');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (72, 'mpg4');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (73, 'ogv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (74, '3gpp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (75, 'mid');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (76, 'midi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (77, 'kar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (78, 'rmi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (79, 'mpga');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (80, 'mp2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (81, 'mp2a');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (83, 'm2a');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (84, 'm3a');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (85, 'sil');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (87, 'weba');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (88, 'woff2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (89, 'exr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (90, 'apng');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (91, 'avci');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (92, 'avcs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (93, 'avif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (94, 'fits');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (95, 'heic');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (96, 'heif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (97, 'hsj2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (98, 'jpeg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (99, 'jpg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (100, 'jpe');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (101, 'jhc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (102, 'jxra');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (103, 'jxrs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (104, 'jxsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (105, 'jxsi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (106, 'jxss');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (107, 'ktx2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (108, 'tif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (109, 'tiff');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (110, 'webp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (111, 'igs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (112, 'iges');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (113, 'msh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (114, 'mesh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (115, 'silo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (116, 'wrl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (117, 'vrml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (118, 'txt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (119, 'text');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (120, 'conf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (121, 'def');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (122, 'list');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (123, 'log');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (124, 'in');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (125, 'ini');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (126, 't');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (127, 'tr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (128, 'roff');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (129, 'man');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (130, 'me');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (131, 'ms');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (132, 'vcard');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (133, 's');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (134, 'asm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (135, 'lua');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (136, 'nfo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (137, 'org');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (138, 'sfv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (139, '3gp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (141, 'h261');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (142, 'h263');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (143, 'h264');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (144, 'jpgv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (145, 'ts');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (146, 'mpeg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (147, 'mpg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (148, 'mpe');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (149, 'm1v');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (150, 'm2v');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (151, 'webm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (152, 'adp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (153, 'au');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (154, 'snd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (155, 'aac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (156, 'caf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (159, 'g3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (160, 'hej2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (161, '3ds');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (162, 'cmx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (163, 'jng');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (164, 'pcx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (165, 'rgb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (166, 'tga');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (167, 'mml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (168, 'stylus');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (169, 'styl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (170, 'ttl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (171, 'opml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (172, 'sass');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (173, 'scss');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (174, '3g2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (175, 'f4v');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (176, 'fli');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (177, 'flv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (178, 'm4v');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (179, 'mng');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (180, 'smv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (181, 'aif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (182, 'aiff');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (183, 'aifc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (184, 'flac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (185, 'ico');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (186, 'pic');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (187, 'pct');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (188, 'fly');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (189, 'vcf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (190, 'dra');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (191, 'dts');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (192, 'rip');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (193, 'pti');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (194, 'svg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (195, 'svgz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (196, 'tfx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (197, 'dwg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (198, 'dxf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (199, 'fpx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (200, 'fst');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (201, 'dwf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (202, 'gdl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (203, 'gtw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (204, 'mts');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (205, 'vtu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (206, 'x3d');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (207, 'x3dz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (208, 'ics');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (209, 'ifb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (210, 'markdown');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (211, 'md');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (212, 'rtx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (213, 'uri');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (214, 'uris');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (215, 'urls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (216, 'curl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (217, 'p');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (218, 'pas');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (219, 'etx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (220, 'fvt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (221, 'wm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (222, 'wax');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (223, 'wma');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (224, 'cdx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (225, 'cif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (226, 'cml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (227, 'xyz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (228, 'btif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (229, 'djvu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (230, 'djv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (231, 'xif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (233, 'u8msg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (234, 'eml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (235, 'mime');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (236, 'stpx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (237, 'stpz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (238, 'x3dv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (239, 'x3dvz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (241, 'f');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (242, 'for');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (243, 'f77');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (244, 'f90');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (245, 'viv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (246, 'asf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (247, 'asx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (248, 'vob');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (249, 'wmv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (250, 'wmx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (251, 'wvx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (252, 'exi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (253, 'gxf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (254, 'mp4s');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (255, 'm4p');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (256, 'mxf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (257, 'oda');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (258, 'ogx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (259, 'pdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (261, 'sdp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (263, 'xsl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (264, 'xsd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (265, 'rng');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (266, 'zip');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (267, 'm3u');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (268, 'cmdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (269, 'csml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (270, 'ttc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (271, 'drle');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (272, 'xbm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (273, 'xpm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (274, 'gltf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (275, 'mkd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (276, 'ymp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (277, 'uu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (278, 'qt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (279, 'mov');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (280, 'avi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (281, 'bdoc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (282, 'gz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (283, 'json');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (284, 'map');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (285, 'mrc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (286, 'mbox');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (287, 'm21');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (288, 'mp21');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (289, 'cjs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (290, 'oxps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (291, 'gram');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (292, 'toml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (293, 'trig');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (294, 'wasm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (295, 'sh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (296, 'xz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (297, 'yang');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (298, 'mxmf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (299, 'dtshd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (300, 'mka');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (301, 'dds');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (302, 'fh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (303, 'fhc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (304, 'fh4');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (305, 'fh5');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (306, 'fh7');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (307, 'x3db');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (308, 'x3dbz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (309, 'wml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (310, 'htc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (311, 'vcs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (312, 'mkv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (313, 'mk3d');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (314, 'mks');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (315, 'hjson');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (316, 'ipfix');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (317, 'json5');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (318, 'p8');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (319, 'siv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (320, 'sieve');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (321, 'arj');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (322, 'cbr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (323, 'cba');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (324, 'cbt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (325, 'cbz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (326, 'cb7');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (327, 'csh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (328, 'dvi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (329, 'eva');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (330, 'hdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (331, 'mie');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (332, 'nzb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (333, 'sea');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (334, 'sql');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (335, 'tar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (336, 'tcl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (337, 'tk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (338, 'tex');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (339, 'ra');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (340, 'mdi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (341, 'npx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (342, 'b16');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (343, 'glb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (344, 'ogex');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (345, 'vds');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (346, 'coffee');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (347, 'litcoffee');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (348, 'gv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (349, 'pde');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (350, 'm4s');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (351, 'uvh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (352, 'uvvh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (353, 'uvp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (354, 'uvvp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (355, 'uvs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (356, 'uvvs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (357, 'mxu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (358, 'm4u');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (359, 'movie');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (360, 'doc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (361, 'dot');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (362, 'p10');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (363, 'ubj');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (364, 'wgt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (365, 'hlp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (367, 'bz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (368, 'chat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (369, 'cpio');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (370, 'wad');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (371, 'gtar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (372, 'pl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (373, 'pm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (374, 'shar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (375, 'gam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (377, 'fig');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (378, 'mxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (379, 'xhvml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (380, 'xvml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (381, 'xvm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (382, 'wdp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (383, 'wbmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (384, 'ras');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (385, 'stpxz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (386, 'usdz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (387, 'dsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (388, '3dml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (389, 'spot');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (390, 'java');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (391, 'dvb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (392, 'uvu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (393, 'uvvu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (394, 'cpl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (395, 'exp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (396, 'fdt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (397, 'gml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (398, 'gpx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (399, 'its');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (400, 'class');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (401, 'jsonld');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (402, 'lgr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (403, 'nq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (404, 'onetoc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (405, 'onetoc2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (406, 'onetmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (407, 'onepkg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (408, 'pki');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (409, 'pls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (410, 'cww');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (411, 'rdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (412, 'owl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (413, 'rsd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (414, 'rss');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (415, 'shf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (416, 'sru');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (417, 'tei');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (418, 'teicorpus');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (419, 'age');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (420, 'bmi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (421, 'dbf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (422, 'dna');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (423, 'fdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (424, 'gmx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (425, 'jam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (426, 'mcd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (427, 'mif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (428, 'rar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (429, 'svd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (430, 'vcx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (431, 'vsf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (432, 'wqd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (433, 'zir');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (434, 'zirz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (435, 'bcpio');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (436, 'blb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (437, 'blorb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (438, 'bz2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (439, 'boz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (440, 'cco');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (441, 'evy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (442, 'ulx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (443, 'latex');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (444, 'prc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (445, 'pdb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (446, 'ustar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (447, 'dtd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (448, 'xop');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (449, 'yin');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (450, 'heics');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (451, 'heifs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (452, 'sid');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (453, 'xwd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (454, 'wsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (455, 'appcache');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (456, 'manifest');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (457, 'dcurl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (458, 'mcurl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (459, 'scurl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (460, 'atom');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (461, 'cdfx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (462, 'cu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (463, 'mpd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (464, 'dssc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (465, 'xdssc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (466, 'emma');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (467, 'epub');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (468, 'geojson');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (469, 'lostxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (470, 'mads');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (471, 'mets');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (472, 'mods');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (473, 'asc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (474, 'crl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (475, 'pskcxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (476, 'roa');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (477, 'sbml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (478, 'smi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (479, 'smil');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (480, 'grxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (481, 'ssdl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (482, 'ssml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (483, 'swidtag');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (484, 'ttml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (485, 'dart');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (486, 'hbci');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (487, 'jisp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (488, 'skp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (489, 'skd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (490, 'skt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (491, 'skm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (492, 'mwf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (493, 'mfm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (494, 'mseq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (495, 'ntf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (496, 'nitf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (498, 'pqa');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (499, 'oprc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (500, 'sema');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (501, 'semd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (502, 'semf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (503, 'mmf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (504, 'ufd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (505, 'ufdl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (506, 'xar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (507, 'xfdl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (508, 'wsdl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (509, 'vcd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (510, 'wmd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (511, 'wmz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (512, 'clp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (513, 'nc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (514, 'cdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (515, 'p12');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (516, 'pfx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (517, 'srt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (518, 'sv4crc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (519, 'xaml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (520, 'xenc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (522, 'xslt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (523, 'xspf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (524, 'uva');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (525, 'uvva');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (526, 'ram');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (529, 'uvv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (530, 'uvvv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (531, 'ccxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (532, 'ink');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (533, 'inkml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (534, 'nt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (535, 'omdoc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (536, 'cer');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (537, 'raml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (538, 'senmlx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (539, 'kne');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (540, 'knp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (541, 'taglet');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (542, 'unityweb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (543, 'vsd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (544, 'vst');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (545, 'vss');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (546, 'vsw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (547, 'abw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (548, 'arc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (549, 'xbap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (550, 'mny');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (551, 'wri');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (552, 'sit');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (553, 'sv4cpio');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (554, 'tfm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (555, 'texinfo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (556, 'texi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (557, 'xhtml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (558, 'xht');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (559, 'xlf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (560, 'xpl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (561, 'tap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (562, 'dae');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (564, 'sub');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (565, 'flx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (566, 'uvm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (567, 'uvvm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (568, 'aw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (569, 'cdmiq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (570, 'es');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (571, 'ecma');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (572, 'pfr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (573, 'js');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (574, 'mjs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (575, 'mathml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (576, 'prf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (577, 'p7m');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (578, 'p7c');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (579, 'ai');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (580, 'eps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (581, 'ps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (582, 'sensmlx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (583, 'tfi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (584, 'gqf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (585, 'gqs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (586, 'hps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (587, 'pcl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (588, 'xpr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (589, 'ims');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (590, 'lrm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (591, 'wpl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (592, 'efif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (593, 'umj');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (594, 'stf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (595, 'dir');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (596, 'dcr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (597, 'dxr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (598, 'cst');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (599, 'cct');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (600, 'cxt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (601, 'w3d');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (602, 'fgd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (603, 'swa');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (604, 'bdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (605, 'pcf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (606, 'snf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (607, 'gnumeric');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (608, 'kdbx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (609, 'run');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (610, 'mdb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (611, 'obd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (612, 'sitx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (613, 'z1');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (614, 'z2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (615, 'z3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (616, 'z4');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (617, 'z5');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (618, 'z6');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (619, 'z7');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (620, 'z8');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (621, 'lvp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (622, 'uvi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (623, 'uvvi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (624, 'uvg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (625, 'uvvg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (627, 'fbs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (628, 'u8hdr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (629, 'wmls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (630, 'atomcat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (631, 'atomsvc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (632, 'cdmid');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (633, 'cdmio');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (634, 'dbk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (635, 'stk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (636, 'jsonml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (637, 'mrcx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (638, 'ma');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (639, 'nb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (640, 'mb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (641, 'maei');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (642, 'musd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (643, 'rif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (644, 'atc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (645, 'acutc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (646, 'dpg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (647, 'ait');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (648, 'geo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (649, 'nml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (650, 'gxt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (651, 'g2w');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (652, 'hal');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (653, 'hpgl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (654, 'hpid');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (655, 'jlt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (656, 'ktz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (657, 'ktr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (658, 'dp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (659, 'cod');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (660, 'see');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (661, 'tra');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (662, 'pgn');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (663, 'php');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (665, 'xpi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (666, 'xel');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (667, 'xns');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (668, 'eol');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (669, 'pnm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (670, 'pbm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (671, 'ppm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (672, 'ice');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (673, 'ez');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (674, 'dwd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (675, 'xcs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (676, 'davmount');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (677, 'jar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (678, 'war');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (679, 'ear');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (680, 'hqx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (681, 'metalink');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (682, 'bin');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (683, 'dms');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (684, 'lrf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (685, 'mar');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (686, 'so');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (687, 'dist');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (688, 'distz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (689, 'pkg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (690, 'bpk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (691, 'dump');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (692, 'elc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (693, 'deploy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (694, 'exe');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (695, 'dll');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (696, 'deb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (697, 'dmg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (698, 'iso');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (699, 'img');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (700, 'msi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (701, 'msp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (702, 'msm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (703, 'buffer');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (704, 'pkipath');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (705, 'rq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (706, 'acu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (707, 'cla');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (708, 'ppd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (709, 'car');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (710, 'uvz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (711, 'uvvz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (712, 'g3w');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (713, 'pclxl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (714, 'igl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (715, 'i2g');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (716, 'qbo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (717, 'qfx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (718, 'fcs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (719, 'xls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (720, 'xlm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (721, 'xla');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (722, 'xlc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (723, 'xlt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (724, 'xlw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (725, 'wps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (726, 'wks');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (727, 'wcm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (728, 'wdb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (729, 'mus');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (730, 'tpt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (731, 'uoml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (732, 'wmlc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (733, 'wtb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (734, 'vxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (735, 'wspolicy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (736, 'torrent');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (737, 'nsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (738, 'ncx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (739, 'dtb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (740, 'pfa');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (741, 'pfb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (742, 'pfm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (743, 'afm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (744, 'gramps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (745, 'crd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (748, 'com');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (749, 'bat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (754, 'emz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (755, 'scd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (756, 'trm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (757, 't3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (758, 'xav');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (760, 'pgm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (761, 'held');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (762, 'rsat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (763, 'emotionml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (764, 'webmanifest');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (765, 'meta4');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (766, 'pgp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (768, 'sig');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (769, 'rapd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (770, 'rusd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (771, 'mft');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (772, 'fxp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (773, 'fxpl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (774, 'ami');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (775, 'pml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (776, 'uvf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (777, 'uvvf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (778, 'uvd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (779, 'uvvd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (780, 'mlp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (781, 'esf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (782, 'msf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (783, 'ssf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (784, 'seed');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (785, 'dataless');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (786, 'flw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (787, 'kwd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (788, 'kwt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (789, 'paw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (790, 'str');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (791, 'ei6');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (792, 'ptid');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (793, 'utz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (794, 'vis');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (795, 'wbxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (796, 'lnk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (797, 'mvb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (798, 'm13');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (799, 'm14');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (800, 'pub');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (801, 'src');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (802, 'xca');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (803, 'xdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (804, 'ecelp4800');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (805, 'ecelp7470');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (806, 'ecelp9600');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (807, 'psd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (808, 'tsv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (809, 'cdmic');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (810, 'mpp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (811, 'cpt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (812, 'ac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (813, 'provx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (814, 'tcap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (815, 'xfdf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (816, 'aep');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (817, 'cdy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (818, 'pcurl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (819, 'slt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (820, 'mseed');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (821, 'gph');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (822, 'fm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (823, 'frame');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (824, 'maker');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (825, 'book');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (826, 'fzs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (827, 'afp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (828, 'listafp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (829, 'list3820');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (830, 'icc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (831, 'icm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (832, 'igm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (833, 'karbon');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (834, 'chrt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (835, 'htke');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (836, 'daf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (837, 'dis');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (838, 'mbk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (839, 'mqy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (840, 'msl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (841, 'plc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (842, 'txf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (843, 'msg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (846, 'mpt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (847, 'xo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (848, 'wg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (849, 'xsm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (850, 'spl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (851, 'luac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (852, 'der');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (853, 'crt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (854, 'pem');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (855, 'hbs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (856, 'atomdeleted');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (857, 'cdmia');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (858, 'relo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (859, 'p7s');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (860, 'scq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (861, 'spq');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (862, 'ahead');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (863, 'pages');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (864, 'rp9');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (865, 'csp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (866, 'cmc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (867, 'kpxx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (868, 'svc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (869, 'ez2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (870, 'fnc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (871, 'ltf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (872, 'ghf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (873, 'mpy');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (874, 'kon');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (875, 'ksp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (876, 'lasxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (877, '123');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (878, 'nsf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (879, 'mc1');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (880, 'cil');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (881, 'chm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (882, 'msty');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (883, 'dd2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (884, 'plf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (885, 'bed');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (886, 'wpd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (887, 'wif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (888, '7z');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (891, 'rmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (892, 'rs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (893, 'sls');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (894, 'scs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (895, 'spp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (896, 'tsd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (897, 'rsheet');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (898, 'azw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (899, 'pkpass');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (900, 'cdxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (901, 'cdbcmsg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (902, 'dfac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (903, 'mag');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (904, 'es3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (905, 'et3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (906, 'vcg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (907, 'kfo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (908, 'kia');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (909, 'nnw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (910, 'edm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (911, 'edx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (912, 'ext');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (913, 'rm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (914, 'dxp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (915, 'sfs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (916, 'wadl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (917, 'sxc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (918, 'sxd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (919, 'sxm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (920, 'sus');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (921, 'susp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (922, 'pcap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (923, 'cap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (924, 'dmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (925, 'mxs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (926, 'ace');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (927, 'aab');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (928, 'x32');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (929, 'u32');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (930, 'vox');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (931, 'aam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (932, 'aas');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (933, 'cfs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (935, 'udeb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (936, 'dgc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (937, 'psf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (938, 'gca');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (939, 'jnlp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (940, 'lzh');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (941, 'lha');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (942, 'application');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (944, 'hdd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (945, 'ova');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (946, 'ovf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (947, 'vdi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (948, 'vhd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (949, 'ged');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (950, 'opf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (951, 'gbr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (952, 'xdp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (953, 'key');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (954, 'm3u8');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (955, 'numbers');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (956, 'c4g');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (957, 'c4d');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (958, 'c4f');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (959, 'c4p');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (960, 'c4u');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (961, 'clkx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (962, 'uvt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (963, 'uvvt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (964, 'ez3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (965, 'ftc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (966, 'fsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (967, 'oas');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (968, 'ddd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (969, 'ggb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (970, 'ggt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (971, 'lwp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (972, 'eot');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (973, 'cat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (974, 'ppt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (975, 'pps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (976, 'pot');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (977, 'pbd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (978, 'teacher');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (979, 'xdm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (980, 'hvd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (982, 'res');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (983, 'swf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (984, 'xap');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (985, 'vbox');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (986, 'vmdk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (987, 'rl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (988, 'srx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (989, 'td');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (990, 'oa2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (991, 'oa3');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (992, 'gac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (993, 'kpr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (994, 'kpt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (995, 'apr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (996, 'flo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (997, 'igx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (998, 'thmx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (999, 'xps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1000, 'esa');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1001, 'cryptonote');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1002, 'sxw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1003, 'tmo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1004, 'wmlsc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1005, 'nbp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1006, 'zaz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1007, 'crx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1008, 'gsf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1010, 'mobi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1011, 'mmr');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1012, 'rlc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1013, 'vtf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1014, 'u8dsn');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1015, 'xer');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1016, 'pvb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1017, 'rep');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1018, 'rdz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1019, 'fg5');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1020, 'grv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1021, 'ivp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1022, 'ivu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1023, 'pre');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1025, 'scm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1026, 'xul');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1027, 'nns');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1028, 'sdkm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1029, 'sdkd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1030, 'sxi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1031, 'sis');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1032, 'sisx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1033, 'bdm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1034, 'hvp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1035, 'jardiff');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1036, 'p7r');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1037, 'pwn');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1038, 'uvx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1039, 'uvvx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1040, 'qam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1041, 'bh2');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1042, 'txd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1043, 'kmz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1044, 'xpw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1045, 'xpx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1046, 'sse');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1047, 'portpkg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1048, 'mgz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1049, 'rmvb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1050, 'ddf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1051, 'hvs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1052, 'p7b');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1053, 'spc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1054, 'pya');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1055, 'azv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1056, 'disposition-notification');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1057, 'jad');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1058, 'pyv');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1059, 'plb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1060, 'psb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1061, 'aso');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1062, 'imp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1063, 'bmml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1064, 'mpm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1065, 'gex');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1066, 'gre');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1067, 'les');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1068, 'cab');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1069, 'nlu');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1070, 'ngdat');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1071, 'obgx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1072, 'qxd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1073, 'qxt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1074, 'qwd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1075, 'qwt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1076, 'qxl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1077, 'qxb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1078, 'sdc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1079, 'sda');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1080, 'smf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1081, 'smzip');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1082, 'saf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1083, 'pac');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1084, 'x_t');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1085, 'ser');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1086, 'mscml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1087, 'setpay');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1088, 'swi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1089, 'joda');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1090, 'mvt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1091, 'cdkey');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1092, 'mpn');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1093, 'mpc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1094, 'nnd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1095, 'rpst');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1096, 'box');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1097, 'mxl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1098, 'link66');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1099, 'twd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1100, 'twds');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1101, 'spf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1102, 'install');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1105, 'rnc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1106, 'rld');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1107, 'mpkg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1108, 'xdw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1109, 'gtm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1111, 'rpss');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1112, 'sdw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1113, 'vor');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1114, 'sm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1115, 'ris');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1116, 'webapp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1117, 'x_b');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1118, 'bsp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1119, 'mpf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1120, 'acc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1121, 'atx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1122, 'mmd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1123, 'gdoc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1124, 'kml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1125, 'tpl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1126, 'sfd-hdstx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1127, 'sc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1128, 'st');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1129, 'sdd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1130, 'rpm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1131, 'azf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1132, 'azs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1133, 'iota');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1134, 'clkp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1135, 'wbs');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1136, 'irm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1137, 'rcprofile');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1138, 'rms');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1139, 'qps');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1140, 'stc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1141, 'std');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1142, 'sxg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1143, 'vbox-extpack');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1144, 'clkk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1145, 'clkt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1146, 'clkw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1147, 'fe_launch');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1148, 'osm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1149, 'mgp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1150, 'musicxml');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1151, 'ipk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1152, 'osf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1154, 'setreg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1155, 'fcdt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1156, 'apk');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1157, 'gsheet');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1158, 'gim');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1159, 'irp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1160, 'odt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1161, 'oxt');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1162, 'ifm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1163, 'stw');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1164, 'cmp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1165, 'u8mdn');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1166, 'csl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1167, 'gslides');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1168, 'odc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1169, 'odi');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1170, 'sti');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1171, 'tao');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1172, 'xbd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1173, 'zmm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1174, 'odf');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1175, 'iif');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1176, 'sgl');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1177, 'odb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1178, 'odg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1179, 'oth');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1180, 'itp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1181, 'fo');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1182, '1km');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1183, 'c11amc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1184, 'n-gage');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1185, 'xlam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1186, 'xlsm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1187, 'ods');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1188, 'odm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1189, 'odp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1190, 'c11amz');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1191, 'docm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1192, 'dotm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1193, 'ott');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1194, 'xltm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1195, 'otc');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1196, 'oti');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1197, 'osfpvg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1198, 'lbd');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1199, 'fti');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1200, 'ppam');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1201, 'sldm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1202, 'odft');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1203, 'otg');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1204, 'xlsb');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1205, 'cii');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1206, 'potm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1207, 'lbe');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1208, 'ppsm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1209, 'ots');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1210, 'otp');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1211, 'pptm');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1212, 'air');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1213, 'xlsx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1214, 'sldx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1215, 'xltx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1216, 'potx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1217, 'ppsx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1218, 'docx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1219, 'dotx');
INSERT INTO omni_mimetypes.file_extensions OVERRIDING SYSTEM VALUE VALUES (1220, 'pptx');


--
-- Data for Name: mime_types; Type: TABLE DATA; Schema: omni_mimetypes; Owner: -
--

INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1, 'text/n3', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2, 'audio/cn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (3, 'audio/dv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (4, 'audio/l8', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (5, 'audio/xm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (6, 'font/otf', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (7, 'font/ttf', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (8, 'text/cmd', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (9, 'text/cql', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (10, 'text/css', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (11, 'text/csv', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (12, 'text/dns', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (13, 'text/jsx', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (14, 'text/mdx', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (15, 'text/red', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (16, 'text/rtf', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (17, 'text/rtx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (18, 'text/vtt', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (19, 'text/x-c', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (20, 'text/xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (21, 'video/dv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (22, 'video/nv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (23, 'audio/aac', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (24, 'audio/ac3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (25, 'audio/amr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (26, 'audio/asc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (27, 'audio/dls', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (28, 'audio/evs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (29, 'audio/gsm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (30, 'audio/l16', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (31, 'audio/l20', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (32, 'audio/l24', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (33, 'audio/lpc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (34, 'audio/mp3', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (35, 'audio/mp4', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (36, 'audio/mpa', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (37, 'audio/ogg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (38, 'audio/red', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (39, 'audio/rtx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (40, 'audio/s3m', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (41, 'audio/smv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (42, 'audio/t38', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (43, 'audio/wav', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (44, 'font/sfnt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (45, 'font/woff', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (46, 'image/bmp', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (47, 'image/cgm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (48, 'image/emf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (49, 'image/gif', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (50, 'image/ief', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (51, 'image/jls', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (52, 'image/jp2', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (53, 'image/jph', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (54, 'image/jpm', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (55, 'image/jpx', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (56, 'image/jxr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (57, 'image/jxs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (58, 'image/ktx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (59, 'image/png', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (60, 'image/sgi', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (61, 'image/t38', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (62, 'image/wmf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (63, 'model/3mf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (64, 'model/e57', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (65, 'model/mtl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (66, 'model/obj', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (67, 'model/stl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (68, 'text/gff3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (69, 'text/html', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (70, 'text/jade', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (71, 'text/less', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (72, 'text/sgml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (73, 'text/shex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (74, 'text/slim', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (75, 'text/spdx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (76, 'text/t140', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (77, 'text/yaml', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (78, 'video/av1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (79, 'video/jpm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (80, 'video/mj2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (81, 'video/mp4', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (82, 'video/mpv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (83, 'video/ogg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (84, 'video/raw', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (85, 'video/rtx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (86, 'video/vc1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (87, 'video/vc2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (88, 'video/vp8', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (89, 'video/vp9', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (90, 'audio/3gpp', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (91, 'audio/aptx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (92, 'audio/bv16', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (93, 'audio/bv32', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (94, 'audio/dvi4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (95, 'audio/eac3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (96, 'audio/evrc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (97, 'audio/g719', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (98, 'audio/g722', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (99, 'audio/g723', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (100, 'audio/g728', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (101, 'audio/g729', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (102, 'audio/ilbc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (103, 'audio/isac', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (104, 'audio/melp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (105, 'audio/mhas', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (106, 'audio/midi', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (107, 'audio/mpeg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (108, 'audio/opus', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (109, 'audio/pcma', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (110, 'audio/pcmu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (111, 'audio/scip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (112, 'audio/silk', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (113, 'audio/smv0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (114, 'audio/sofa', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (115, 'audio/tone', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (116, 'audio/usac', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (117, 'audio/vdvi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (118, 'audio/wave', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (119, 'audio/webm', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (120, 'font/woff2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (121, 'image/aces', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (122, 'image/apng', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (123, 'image/avci', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (124, 'image/avcs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (125, 'image/avif', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (126, 'image/fits', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (127, 'image/heic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (128, 'image/heif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (129, 'image/hsj2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (130, 'image/jpeg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (131, 'image/jphc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (132, 'image/jxra', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (133, 'image/jxrs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (134, 'image/jxsc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (135, 'image/jxsi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (136, 'image/jxss', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (137, 'image/ktx2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (138, 'image/tiff', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (139, 'image/webp', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (140, 'model/iges', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (141, 'model/mesh', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (142, 'model/step', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (143, 'model/vrml', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (144, 'text/mizar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (145, 'text/plain', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (146, 'text/troff', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (147, 'text/vcard', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (148, 'text/vnd.a', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (149, 'text/x-asm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (150, 'text/x-lua', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (151, 'text/x-nfo', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (152, 'text/x-org', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (153, 'text/x-sfv', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (154, 'video/3gpp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (155, 'video/celb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (156, 'video/ffv1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (157, 'video/h261', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (158, 'video/h263', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (159, 'video/h264', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (160, 'video/h265', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (161, 'video/jpeg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (162, 'video/jxsv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (163, 'video/mp1s', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (164, 'video/mp2p', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (165, 'video/mp2t', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (166, 'video/mpeg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (167, 'video/scip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (168, 'video/webm', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (169, 'audio/3gpp2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (170, 'audio/adpcm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (171, 'audio/basic', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (172, 'audio/dat12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (173, 'audio/evrc0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (174, 'audio/evrc1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (175, 'audio/evrcb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (176, 'audio/g7221', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (177, 'audio/g7291', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (178, 'audio/g729d', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (179, 'audio/g729e', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (180, 'audio/qcelp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (181, 'audio/speex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (182, 'audio/t140c', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (183, 'audio/x-aac', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (184, 'audio/x-caf', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (185, 'audio/x-m4a', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (186, 'audio/x-tta', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (187, 'audio/x-wav', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (188, 'image/g3fax', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (189, 'image/hej2k', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (190, 'image/pjpeg', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (191, 'image/x-3ds', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (192, 'image/x-cmx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (193, 'image/x-jng', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (194, 'image/x-pcx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (195, 'image/x-rgb', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (196, 'image/x-tga', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (197, 'image/x-xcf', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (198, 'message/sip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (199, 'text/fwdred', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (200, 'text/mathml', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (201, 'text/shaclc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (202, 'text/stylus', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (203, 'text/turtle', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (204, 'text/ulpfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (205, 'text/x-opml', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (206, 'text/x-sass', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (207, 'text/x-scss', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (208, 'video/3gpp2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (209, 'video/bmpeg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (210, 'video/bt656', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (211, 'video/x-f4v', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (212, 'video/x-fli', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (213, 'video/x-flv', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (214, 'video/x-m4v', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (215, 'video/x-mng', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (216, 'video/x-smv', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (217, 'audio/amr-wb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (218, 'audio/atrac3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (219, 'audio/evrcb0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (220, 'audio/evrcb1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (221, 'audio/evrcnw', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (222, 'audio/evrcwb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (223, 'audio/fwdred', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (224, 'audio/g711-0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (225, 'audio/tsvcis', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (226, 'audio/ulpfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (227, 'audio/vmr-wb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (228, 'audio/vorbis', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (229, 'audio/x-aiff', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (230, 'audio/x-flac', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (231, 'image/naplps', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (232, 'image/x-icon', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (233, 'image/x-pict', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (234, 'message/cpim', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (235, 'message/http', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (236, 'message/news', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (237, 'text/flexfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (238, 'text/jcr-cnd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (239, 'text/strings', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (240, 'text/vnd.abc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (241, 'text/vnd.fly', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (242, 'text/vnd.gml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (243, 'text/vnd.hgl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (244, 'text/x-vcard', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (245, 'video/ulpfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (246, 'audio/amr-wb+', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (247, 'audio/atrac-x', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (248, 'audio/evrcnw0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (249, 'audio/evrcnw1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (250, 'audio/evrcwb0', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (251, 'audio/evrcwb1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (252, 'audio/flexfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (253, 'audio/g726-16', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (254, 'audio/g726-24', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (255, 'audio/g726-32', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (256, 'audio/g726-40', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (257, 'audio/gsm-efr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (258, 'audio/melp600', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (259, 'audio/pcma-wb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (260, 'audio/pcmu-wb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (261, 'audio/prs.sid', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (262, 'audio/smv-qcp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (263, 'audio/sp-midi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (264, 'audio/uemclip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (265, 'audio/vnd.4sb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (266, 'audio/vnd.dra', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (267, 'audio/vnd.dts', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (268, 'audio/vnd.rip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (269, 'image/prs.pti', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (270, 'image/svg+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (271, 'image/tiff-fx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (272, 'image/vnd.dwg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (273, 'image/vnd.dxf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (274, 'image/vnd.fpx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (275, 'image/vnd.fst', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (276, 'image/vnd.mix', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (277, 'image/vnd.svf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (278, 'model/vnd.dwf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (279, 'model/vnd.gdl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (280, 'model/vnd.gtw', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (281, 'model/vnd.mts', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (282, 'model/vnd.vtu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (283, 'model/x3d+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (284, 'text/calendar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (285, 'text/calender', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (286, 'text/encaprtp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (287, 'text/enriched', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (288, 'text/fhirpath', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (289, 'text/markdown', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (290, 'text/richtext', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (291, 'text/uri-list', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (292, 'text/vnd.curl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (293, 'text/vnd.hans', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (294, 'text/vnd.sosi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (295, 'text/x-pascal', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (296, 'text/x-setext', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (297, 'video/3gpp-tt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (298, 'video/flexfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (299, 'video/mp4v-es', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (300, 'video/pointer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (301, 'video/vnd.fvt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (302, 'video/x-ms-wm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (303, 'audio/32kadpcm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (304, 'audio/encaprtp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (305, 'audio/evrc-qcp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (306, 'audio/melp1200', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (307, 'audio/melp2400', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (308, 'audio/musepack', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (309, 'audio/rtp-midi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (310, 'audio/vnd.celp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (311, 'audio/vnd.wave', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (312, 'audio/x-ms-wax', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (313, 'audio/x-ms-wma', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (314, 'chemical/x-cdx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (315, 'chemical/x-cif', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (316, 'chemical/x-cml', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (317, 'chemical/x-pdb', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (318, 'chemical/x-xyz', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (319, 'image/prs.btif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (320, 'image/vnd.djvu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (321, 'image/vnd.xiff', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (322, 'image/x-ms-bmp', 'nginx', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (323, 'message/global', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (324, 'message/rfc822', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (325, 'message/s-http', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (326, 'model/step+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (327, 'model/step+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (328, 'model/x3d+vrml', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (329, 'model/x3d-vrml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (330, 'text/directory', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (331, 'text/parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (332, 'text/raptorfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (333, 'text/x-fortran', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (334, 'text/x-gwt-rpc', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (335, 'video/encaprtp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (336, 'video/h264-svc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (337, 'video/jpeg2000', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (338, 'video/smpte291', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (339, 'video/vnd.cctv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (340, 'video/vnd.vivo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (341, 'video/x-ms-asf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (342, 'video/x-ms-vob', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (343, 'video/x-ms-wmv', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (344, 'video/x-ms-wmx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (345, 'video/x-ms-wvx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (346, 'application/a2l', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (347, 'application/aml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (348, 'application/atf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (349, 'application/cea', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (350, 'application/cfw', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (351, 'application/clr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (352, 'application/cms', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (353, 'application/cwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (354, 'application/dcd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (355, 'application/dii', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (356, 'application/dit', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (357, 'application/dns', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (358, 'application/efi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (359, 'application/exi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (360, 'application/gxf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (361, 'application/ipp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (362, 'application/jwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (363, 'application/lxf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (364, 'application/mf4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (365, 'application/mp4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (366, 'application/mxf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (367, 'application/nss', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (368, 'application/oda', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (369, 'application/odx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (370, 'application/ogg', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (371, 'application/p21', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (372, 'application/pdf', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (373, 'application/pdx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (374, 'application/rtf', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (375, 'application/rtx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (376, 'application/sbe', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (377, 'application/sdp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (378, 'application/sql', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (379, 'application/tar', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (380, 'application/xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (381, 'application/zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (382, 'audio/clearmode', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (383, 'audio/gsm-hr-08', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (384, 'audio/mp4a-latm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (385, 'audio/parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (386, 'audio/raptorfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (387, 'audio/vnd.qcelp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (388, 'audio/x-mpegurl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (389, 'chemical/x-cmdf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (390, 'chemical/x-csml', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (391, 'font/collection', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (392, 'image/dicom-rle', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (393, 'image/x-xbitmap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (394, 'image/x-xpixmap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (395, 'message/partial', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (396, 'message/sipfrag', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (397, 'model/gltf+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (398, 'multipart/mixed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (399, 'text/csv-schema', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (400, 'text/ecmascript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (401, 'text/javascript', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (402, 'text/parameters', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (403, 'text/vnd.wap.si', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (404, 'text/vnd.wap.sl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (405, 'text/x-markdown', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (406, 'text/x-suse-ymp', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (407, 'text/x-uuencode', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (408, 'video/h263-1998', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (409, 'video/h263-2000', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (410, 'video/h264-rcdo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (411, 'video/parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (412, 'video/quicktime', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (413, 'video/raptorfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (414, 'video/smpte292m', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (415, 'video/x-msvideo', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (416, 'application/atfx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (417, 'application/bdoc', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (418, 'application/cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (419, 'application/cdni', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (420, 'application/cose', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (421, 'application/dart', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (422, 'application/dvcs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (423, 'application/fits', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (424, 'application/gzip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (425, 'application/h224', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (426, 'application/http', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (427, 'application/iges', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (428, 'application/iotp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (429, 'application/isup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (430, 'application/jose', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (431, 'application/json', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (432, 'application/marc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (433, 'application/mbox', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (434, 'application/mipc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (435, 'application/mp21', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (436, 'application/node', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (437, 'application/oxps', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (438, 'application/qsig', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (439, 'application/sgml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (440, 'application/sipc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (441, 'application/smil', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (442, 'application/srgs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (443, 'application/toml', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (444, 'application/trig', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (445, 'application/tzif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (446, 'application/wasm', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (447, 'application/wita', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (448, 'application/x-sh', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (449, 'application/x-xz', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (450, 'application/yang', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (451, 'application/zlib', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (452, 'application/zstd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (453, 'audio/ip-mr_v2.5', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (454, 'audio/mobile-xmf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (455, 'audio/mpa-robust', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (456, 'audio/vnd.dts.hd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (457, 'audio/x-matroska', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (458, 'image/pwg-raster', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (459, 'image/vnd.ms-dds', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (460, 'image/x-freehand', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (461, 'message/imdn+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (462, 'model/vnd.gs-gdl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (463, 'model/vnd.gs.gdl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (464, 'model/x3d+binary', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (465, 'multipart/digest', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (466, 'multipart/report', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (467, 'multipart/signed', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (468, 'text/rtploopback', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (469, 'text/vnd.latex-z', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (470, 'text/vnd.wap.wml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (471, 'text/x-component', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (472, 'text/x-vcalendar', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (473, 'video/x-matroska', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (474, 'application/atxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (475, 'application/cccex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (476, 'application/dicom', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (477, 'application/eshop', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (478, 'application/hjson', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (479, 'application/index', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (480, 'application/ipfix', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (481, 'application/json5', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (482, 'application/mikey', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (483, 'application/pkcs8', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (484, 'application/sieve', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (485, 'application/slate', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (486, 'application/vemmi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (487, 'application/x-amf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (488, 'application/x-arj', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (489, 'application/x-cbr', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (490, 'application/x-csh', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (491, 'application/x-deb', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (492, 'application/x-dvi', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (493, 'application/x-eva', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (494, 'application/x-hdf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (495, 'application/x-mie', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (496, 'application/x-nzb', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (497, 'application/x-sea', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (498, 'application/x-sql', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (499, 'application/x-tar', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (500, 'application/x-tcl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (501, 'application/x-tex', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (502, 'audio/rtploopback', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (503, 'audio/tetra_acelp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (504, 'audio/vnd.dts.uhd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (505, 'audio/x-realaudio', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (506, 'image/vnd.ms-modi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (507, 'image/vnd.net-fpx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (508, 'image/vnd.pco.b16', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (509, 'model/gltf-binary', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (510, 'model/vnd.opengex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (511, 'model/vnd.sap.vds', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (512, 'multipart/related', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (513, 'text/coffeescript', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (514, 'text/vnd.graphviz', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (515, 'text/x-processing', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (516, 'video/iso.segment', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (517, 'video/rtploopback', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (518, 'video/vnd.dece.hd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (519, 'video/vnd.dece.pd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (520, 'video/vnd.dece.sd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (521, 'video/vnd.mpegurl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (522, 'video/x-sgi-movie', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (523, 'x-shader/x-vertex', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (524, 'application/at+jwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (525, 'application/dec-dx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (526, 'application/msword', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (527, 'application/oscore', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (528, 'application/pkcs10', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (529, 'application/pkcs12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (530, 'application/riscos', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (531, 'application/ubjson', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (532, 'application/ulpfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (533, 'application/widget', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (534, 'application/winhlp', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (535, 'application/x-bdoc', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (536, 'application/x-bzip', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (537, 'application/x-chat', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (538, 'application/x-cpio', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (539, 'application/x-doom', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (540, 'application/x-gtar', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (541, 'application/x-gzip', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (542, 'application/x-perl', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (543, 'application/x-shar', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (544, 'application/x-tads', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (545, 'application/x-tgif', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (546, 'application/x-xfig', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (547, 'application/xv+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (548, 'audio/dsr-es201108', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (549, 'audio/dsr-es202050', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (550, 'audio/dsr-es202211', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (551, 'audio/dsr-es202212', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (552, 'audio/vnd.audiokoz', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (553, 'audio/vnd.cns.anp1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (554, 'audio/vnd.cns.inf1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (555, 'audio/vnd.dvb.file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (556, 'audio/vnd.vmx.cvsd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (557, 'image/vnd.cns.inf2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (558, 'image/vnd.ms-photo', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (559, 'image/vnd.radiance', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (560, 'image/vnd.wap.wbmp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (561, 'image/x-cmu-raster', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (562, 'model/step-xml+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (563, 'model/vnd.moml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (564, 'model/vnd.usdz+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (565, 'multipart/parallel', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (566, 'text/prs.lines.tag', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (567, 'text/vnd.ascii-art', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (568, 'text/vnd.in3d.3dml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (569, 'text/vnd.in3d.spot', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (570, 'text/vnd.iptc.nitf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (571, 'text/x-java-source', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (572, 'text/x-jquery-tmpl', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (573, 'video/vnd.dece.mp4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (574, 'video/vnd.dvb.file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (575, 'video/vnd.uvvu.mp4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (576, 'application/cpl+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (577, 'application/dca-rft', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (578, 'application/edi-x12', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (579, 'application/edifact', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (580, 'application/elm+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (581, 'application/epp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (582, 'application/express', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (583, 'application/fdt+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (584, 'application/flexfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (585, 'application/gml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (586, 'application/gpx+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (587, 'application/its+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (588, 'application/java-vm', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (589, 'application/ld+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (590, 'application/lgr+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (591, 'application/lpf+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (592, 'application/n-quads', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (593, 'application/nasdata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (594, 'application/odm+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (595, 'application/onenote', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (596, 'application/p21+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (597, 'application/pkixcmp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (598, 'application/pls+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (599, 'application/prs.cww', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (600, 'application/prs.cyn', 'iana', NULL, '7-BIT');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (601, 'application/rdf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (602, 'application/rfc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (603, 'application/rsd+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (604, 'application/rss+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (605, 'application/sep+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (606, 'application/sep-exi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (607, 'application/shf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (608, 'application/sru+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (609, 'application/td+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (610, 'application/tei+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (611, 'application/vnd.age', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (612, 'application/vnd.bmi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (613, 'application/vnd.bpf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (614, 'application/vnd.dbf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (615, 'application/vnd.dna', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (616, 'application/vnd.dxr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (617, 'application/vnd.dzr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (618, 'application/vnd.fdf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (619, 'application/vnd.gmx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (620, 'application/vnd.hdt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (621, 'application/vnd.jam', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (622, 'application/vnd.las', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (623, 'application/vnd.mcd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (624, 'application/vnd.mif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (625, 'application/vnd.obn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (626, 'application/vnd.rar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (627, 'application/vnd.sar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (628, 'application/vnd.shp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (629, 'application/vnd.shx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (630, 'application/vnd.svd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (631, 'application/vnd.tml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (632, 'application/vnd.vcx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (633, 'application/vnd.vsf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (634, 'application/vnd.wmc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (635, 'application/vnd.wqd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (636, 'application/vnd.zul', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (637, 'application/x-bcpio', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (638, 'application/x-blorb', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (639, 'application/x-bzip2', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (640, 'application/x-cocoa', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (641, 'application/x-envoy', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (642, 'application/x-glulx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (643, 'application/x-latex', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (644, 'application/x-pilot', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (645, 'application/x-ustar', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (646, 'application/x400-bp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (647, 'application/xml-dtd', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (648, 'application/xop+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (649, 'application/yin+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (650, 'audio/mpeg4-generic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (651, 'audio/vnd.3gpp.iufp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (652, 'audio/vnd.cisco.nse', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (653, 'audio/vnd.dlna.adts', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (654, 'audio/vnd.dolby.mlp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (655, 'audio/vnd.dolby.mps', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (656, 'audio/vnd.dolby.pl2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (657, 'audio/vnd.hns.audio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (658, 'audio/vnd.octel.sbc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (659, 'audio/vorbis-config', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (660, 'image/heic-sequence', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (661, 'image/heif-sequence', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (662, 'image/x-mrsid-image', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (663, 'image/x-xwindowdump', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (664, 'message/vnd.si.simp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (665, 'message/vnd.wfa.wsc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (666, 'multipart/encrypted', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (667, 'multipart/form-data', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (668, 'text/cache-manifest', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (669, 'text/cql-expression', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (670, 'text/cql-identifier', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (671, 'text/prs.prop.logic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (672, 'text/rfc822-headers', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (673, 'text/vnd.curl.dcurl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (674, 'text/vnd.curl.mcurl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (675, 'text/vnd.curl.scurl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (676, 'text/vnd.ficlab.flt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (677, 'video/mpeg4-generic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (678, 'video/vnd.hns.video', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (679, 'x-shader/x-fragment', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (680, 'application/ace+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (681, 'application/atom+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (682, 'application/beep+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (683, 'application/cbor-seq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (684, 'application/ccmp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (685, 'application/cdfx+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (686, 'application/clue+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (687, 'application/cnrp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (688, 'application/cose-key', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (689, 'application/csrattrs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (690, 'application/csta+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (691, 'application/cu-seeme', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (692, 'application/dash+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (693, 'application/dns+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (694, 'application/dssc+der', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (695, 'application/dssc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (696, 'application/elm+json', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (697, 'application/emma+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (698, 'application/encaprtp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (699, 'application/epub+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (700, 'application/fastsoap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (701, 'application/fhir+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (702, 'application/geo+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (703, 'application/held+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (704, 'application/jrd+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (705, 'application/json-seq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (706, 'application/jwk+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (707, 'application/lost+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (708, 'application/mads+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (709, 'application/mets+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (710, 'application/mods+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (711, 'application/mud+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (712, 'application/passport', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (713, 'application/pgp-keys', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (714, 'application/pidf+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (715, 'application/pkix-crl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (716, 'application/pskc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (717, 'application/pvd+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (718, 'application/rlmi+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (719, 'application/rpki-roa', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (720, 'application/sbml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (721, 'application/smil+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (722, 'application/soap+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (723, 'application/srgs+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (724, 'application/ssdl+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (725, 'application/ssml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (726, 'application/swid+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (727, 'application/ttml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (728, 'application/vnd.anki', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (729, 'application/vnd.bpf3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (730, 'application/vnd.curl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (731, 'application/vnd.dart', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (732, 'application/vnd.hbci', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (733, 'application/vnd.jisp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (734, 'application/vnd.koan', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (735, 'application/vnd.loom', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (736, 'application/vnd.mfer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (737, 'application/vnd.mfmp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (738, 'application/vnd.mseq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (739, 'application/vnd.nimn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (740, 'application/vnd.nitf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (741, 'application/vnd.palm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (742, 'application/vnd.pcos', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (743, 'application/vnd.psfs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (744, 'application/vnd.sema', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (745, 'application/vnd.semd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (746, 'application/vnd.semf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (747, 'application/vnd.smaf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (748, 'application/vnd.ufdl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (749, 'application/vnd.xara', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (750, 'application/vnd.xfdl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (751, 'application/wsdl+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (752, 'application/x-cdlink', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (753, 'application/x-ms-wmd', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (754, 'application/x-ms-wmz', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (755, 'application/x-msclip', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (756, 'application/x-netcdf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (757, 'application/x-pkcs12', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (758, 'application/x-subrip', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (759, 'application/x-sv4crc', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (760, 'application/xaml+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (761, 'application/xenc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (762, 'application/xmpp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (763, 'application/xslt+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (764, 'application/xspf+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (765, 'audio/tetra_acelp_bb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (766, 'audio/vnd.dece.audio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (767, 'audio/vnd.dolby.pl2x', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (768, 'audio/vnd.dolby.pl2z', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (769, 'audio/vnd.everad.plj', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (770, 'audio/vnd.nortel.vbk', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (771, 'audio/x-pn-realaudio', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (772, 'image/vnd.sealed.png', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (773, 'image/vnd.zbrush.pcx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (774, 'model/vnd.pytha.pyox', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (775, 'multipart/byteranges', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (776, 'multipart/header-set', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (777, 'text/vnd.iptc.newsml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (778, 'video/vnd.dece.video', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (779, 'video/vnd.sealed.swf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (780, 'video/vnd.youtube.yt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (781, 'application/applefile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (782, 'application/cals-1840', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (783, 'application/ccxml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (784, 'application/city+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (785, 'application/csvm+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (786, 'application/cybercash', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (787, 'application/dashdelta', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (788, 'application/dicom+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (789, 'application/dots+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (790, 'application/dskpp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (791, 'application/fhir+json', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (792, 'application/font-sfnt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (793, 'application/font-woff', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (794, 'application/index.cmd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (795, 'application/index.obj', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (796, 'application/index.vnd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (797, 'application/inkml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (798, 'application/jose+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (799, 'application/moss-keys', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (800, 'application/mpeg4-iod', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (801, 'application/n-triples', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (802, 'application/nlsml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (803, 'application/omdoc+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (804, 'application/parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (805, 'application/pkix-cert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (806, 'application/raml+yaml', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (807, 'application/raptorfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (808, 'application/rdap+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (809, 'application/scaip+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (810, 'application/scim+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (811, 'application/senml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (812, 'application/senml-exi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (813, 'application/smpte336m', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (814, 'application/spdx+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (815, 'application/stix+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (816, 'application/tetra_isi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (817, 'application/tzif-leap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (818, 'application/vcard+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (819, 'application/vnd.ciedi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (820, 'application/vnd.ffsns', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (821, 'application/vnd.fints', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (822, 'application/vnd.kinar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (823, 'application/vnd.msign', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (824, 'application/vnd.mynfc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (825, 'application/vnd.rapid', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (826, 'application/vnd.s3sms', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (827, 'application/vnd.unity', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (828, 'application/vnd.visio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (829, 'application/vq-rtcpxr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (830, 'application/x-abiword', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (831, 'application/x-freearc', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (832, 'application/x-mpegurl', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (833, 'application/x-ms-xbap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (834, 'application/x-msmoney', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (835, 'application/x-mswrite', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (836, 'application/x-stuffit', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (837, 'application/x-sv4cpio', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (838, 'application/x-tex-tfm', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (839, 'application/x-texinfo', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (840, 'application/xacml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (841, 'application/xhtml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (842, 'application/xliff+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (843, 'application/xproc+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (844, 'audio/telephone-event', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (845, 'image/vnd.tencent.tap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (846, 'message/external-body', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (847, 'model/vnd.collada+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (848, 'model/x3d+fastinfoset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (849, 'multipart/alternative', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (850, 'multipart/appledouble', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (851, 'text/grammar-ref-list', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (852, 'text/rtp-enc-aescm128', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (853, 'text/vnd.dvb.subtitle', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (854, 'text/vnd.fmi.flexstor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (855, 'video/vnd.dece.mobile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (856, 'video/vnd.nokia.mp4vr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (857, 'video/vnd.objectvideo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (858, 'application/applixware', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (859, 'application/atomicmail', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (860, 'application/batch-smtp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (861, 'application/cdmi-queue', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (862, 'application/cellml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (863, 'application/dicom+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (864, 'application/ecmascript', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (865, 'application/font-tdpfr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (866, 'application/javascript', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (867, 'application/macwriteii', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (868, 'application/mathml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (869, 'application/pics-rules', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (870, 'application/pkcs7-mime', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (871, 'application/postscript', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (872, 'application/prs.nprend', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (873, 'application/sarif+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (874, 'application/senml+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (875, 'application/senml+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (876, 'application/sensml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (877, 'application/sensml-exi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (878, 'application/tamp-error', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (879, 'application/taxii+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (880, 'application/thraud+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (881, 'application/tnauthlist', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (882, 'application/vcard+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (883, 'application/vnd.cybank', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (884, 'application/vnd.gerber', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (885, 'application/vnd.grafeq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (886, 'application/vnd.gridmp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (887, 'application/vnd.hp-hps', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (888, 'application/vnd.hp-pcl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (889, 'application/vnd.is-xpr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (890, 'application/vnd.laszip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (891, 'application/vnd.ms-asf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (892, 'application/vnd.ms-ims', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (893, 'application/vnd.ms-lrm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (894, 'application/vnd.ms-wpl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (895, 'application/vnd.netfpx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (896, 'application/vnd.picsel', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (897, 'application/vnd.umajin', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (898, 'application/vnd.wt.stf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (899, 'application/x-compress', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (900, 'application/x-director', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (901, 'application/x-font-bdf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (902, 'application/x-font-dos', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (903, 'application/x-font-pcf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (904, 'application/x-font-snf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (905, 'application/x-gnumeric', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (906, 'application/x-keepass2', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (907, 'application/x-makeself', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (908, 'application/x-msaccess', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (909, 'application/x-msbinder', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (910, 'application/x-stuffitx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (911, 'application/x-zmachine', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (912, 'audio/rtp-enc-aescm128', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (913, 'audio/vnd.lucent.voice', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (914, 'audio/vnd.rn-realaudio', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (915, 'image/vnd.dece.graphic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (916, 'image/vnd.dvb.subtitle', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (917, 'image/vnd.fastbidsheet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (918, 'image/vnd.mozilla.apng', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (919, 'message/global-headers', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (920, 'multipart/multilingual', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (921, 'text/vnd.wap.wmlscript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (922, 'video/rtp-enc-aescm128', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (923, 'video/vnd.directv.mpeg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (924, 'video/vnd.sealed.mpeg1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (925, 'video/vnd.sealed.mpeg4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (926, 'application/atomcat+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (927, 'application/atomsvc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (928, 'application/cdmi-domain', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (929, 'application/cdmi-object', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (930, 'application/dns-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (931, 'application/docbook+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (932, 'application/edi-consent', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (933, 'application/fastinfoset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (934, 'application/gltf-buffer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (935, 'application/hyperstudio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (936, 'application/ibe-pp-data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (937, 'application/jsonml+json', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (938, 'application/link-format', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (939, 'application/marcxml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (940, 'application/mathematica', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (941, 'application/mmt-aei+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (942, 'application/mmt-usd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (943, 'application/msc-ivr+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (944, 'application/problem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (945, 'application/prs.plucker', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (946, 'application/prs.xsf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (947, 'application/reginfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (948, 'application/rpki-updown', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (949, 'application/rtploopback', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (950, 'application/sensml+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (951, 'application/sensml+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (952, 'application/set-payment', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (953, 'application/tamp-update', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (954, 'application/tlsrpt+gzip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (955, 'application/tlsrpt+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (956, 'application/tve-trigger', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (957, 'application/vnd.acucorp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (958, 'application/vnd.dpgraph', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (959, 'application/vnd.dvb.ait', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (960, 'application/vnd.dvb.pfr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (961, 'application/vnd.dynageo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (962, 'application/vnd.efi.img', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (963, 'application/vnd.efi.iso', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (964, 'application/vnd.enliven', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (965, 'application/vnd.geonext', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (966, 'application/vnd.geoplan', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (967, 'application/vnd.hal+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (968, 'application/vnd.hc+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (969, 'application/vnd.hp-hpgl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (970, 'application/vnd.hp-hpid', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (971, 'application/vnd.hp-jlyt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (972, 'application/vnd.kahootz', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (973, 'application/vnd.ms-tnef', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (974, 'application/vnd.nervana', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (975, 'application/vnd.oma.dcd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (976, 'application/vnd.osgi.dp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (977, 'application/vnd.panoply', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (978, 'application/vnd.rim.cod', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (979, 'application/vnd.rs-274x', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (980, 'application/vnd.sbm.cid', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (981, 'application/vnd.scribus', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (982, 'application/vnd.seemail', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (983, 'application/vnd.sqlite3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (984, 'application/vnd.sss-cod', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (985, 'application/vnd.sss-dtf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (986, 'application/vnd.sss-ntf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (987, 'application/vnd.trueapp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (988, 'application/vnd.truedoc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (989, 'application/vnd.uri-map', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (990, 'application/vnd.wap.sic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (991, 'application/vnd.wap.slc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (992, 'application/vnd.wfa.dpp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (993, 'application/vnd.wfa.p2p', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (994, 'application/vnd.wfa.wsc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (995, 'application/vnd.xmi+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (996, 'application/vnd.yaoweme', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (997, 'application/x-chess-pgn', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (998, 'application/x-httpd-php', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (999, 'application/x-xliff+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1000, 'application/x-xpinstall', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1001, 'application/xcap-el+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1002, 'application/xcap-ns+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1003, 'audio/vnd.digital-winds', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1004, 'audio/vnd.dolby.heaac.1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1005, 'audio/vnd.dolby.heaac.2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1006, 'audio/vnd.dolby.pulse.1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1007, 'image/x-portable-anymap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1008, 'image/x-portable-bitmap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1009, 'image/x-portable-pixmap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1010, 'message/delivery-status', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1011, 'message/feedback-report', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1012, 'message/tracking-status', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1013, 'model/vnd.flatland.3dml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1014, 'multipart/voice-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1015, 'text/vnd.dmclientscript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1016, 'video/vnd.dlna.mpeg-tts', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1017, 'x-conference/x-cooltalk', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1018, 'application/3gpp-ims+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1019, 'application/3gpphal+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1020, 'application/andrew-inset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1021, 'application/atsc-dwd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1022, 'application/calendar+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1023, 'application/captive+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1024, 'application/cea-2018+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1025, 'application/coap-payload', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1026, 'application/commonground', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1027, 'application/cose-key-set', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1028, 'application/cstadata+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1029, 'application/davmount+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1030, 'application/geo+json-seq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1031, 'application/geoxacml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1032, 'application/java-archive', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1033, 'application/jf2feed+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1034, 'application/jwk-set+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1035, 'application/lostsync+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1036, 'application/mac-binhex40', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1037, 'application/mbms-msk+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1038, 'application/metalink+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1039, 'application/mosskey-data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1040, 'application/ocsp-request', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1041, 'application/octet-stream', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1042, 'application/pkix-pkipath', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1043, 'application/problem+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1044, 'application/prs.hpub+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1045, 'application/reputon+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1046, 'application/secevent+jwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1047, 'application/session-info', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1048, 'application/sparql-query', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1049, 'application/vnd.3gpp.lpp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1050, 'application/vnd.3gpp.sms', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1051, 'application/vnd.acucobol', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1052, 'application/vnd.api+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1053, 'application/vnd.claymore', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1054, 'application/vnd.cups-pdf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1055, 'application/vnd.cups-ppd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1056, 'application/vnd.cups-raw', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1057, 'application/vnd.curl.car', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1058, 'application/vnd.dece.zip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1059, 'application/vnd.dvb.dvbj', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1060, 'application/vnd.ecip.rlp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1061, 'application/vnd.geo+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1062, 'application/vnd.geospace', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1063, 'application/vnd.hal+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1064, 'application/vnd.hp-pclxl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1065, 'application/vnd.httphone', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1066, 'application/vnd.igloader', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1067, 'application/vnd.intergeo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1068, 'application/vnd.intu.qbo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1069, 'application/vnd.intu.qfx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1070, 'application/vnd.isac.fcs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1071, 'application/vnd.ms-excel', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1072, 'application/vnd.ms-works', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1073, 'application/vnd.musician', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1074, 'application/vnd.ocf+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1075, 'application/vnd.oma.dcdc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1076, 'application/vnd.oma.push', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1077, 'application/vnd.onepager', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1078, 'application/vnd.paos.xml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1079, 'application/vnd.preminet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1080, 'application/vnd.sbm.mid2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1081, 'application/vnd.trid.tpt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1082, 'application/vnd.uoml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1083, 'application/vnd.vd-study', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1084, 'application/vnd.vel+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1085, 'application/vnd.wap.wmlc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1086, 'application/vnd.webturbo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1087, 'application/voicexml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1088, 'application/wspolicy+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1089, 'application/x-bittorrent', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1090, 'application/x-conference', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1091, 'application/x-dtbncx+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1092, 'application/x-dtbook+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1093, 'application/x-font-type1', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1094, 'application/x-font-vfont', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1095, 'application/x-gramps-xml', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1096, 'application/x-javascript', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1097, 'application/x-mscardfile', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1098, 'application/x-msdownload', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1099, 'application/x-msmetafile', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1100, 'application/x-msschedule', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1101, 'application/x-msterminal', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1102, 'application/x-t3vm-image', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1103, 'application/xcap-att+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1104, 'image/vnd.microsoft.icon', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1105, 'image/x-portable-graymap', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1106, 'text/provenance-notation', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1107, 'text/prs.fallenstein.rst', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1108, 'text/vnd.motorola.reflex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1109, 'text/vnd.ms-mediapackage', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1110, 'text/vnd.senx.warpscript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1111, 'text/vnd.si.uricatalogue', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1112, 'video/vnd.motorola.video', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1113, 'application/activemessage', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1114, 'application/activity+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1115, 'application/atsc-held+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1116, 'application/atsc-rdt+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1117, 'application/atsc-rsat+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1118, 'application/calendar+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1119, 'application/clue_info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1120, 'application/emotionml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1121, 'application/manifest+json', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1122, 'application/metalink4+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1123, 'application/mpeg4-generic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1124, 'application/mpeg4-iod-xmt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1125, 'application/msc-mixer+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1126, 'application/ocsp-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1127, 'application/pgp-encrypted', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1128, 'application/pgp-signature', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1129, 'application/pidf-diff+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1130, 'application/route-apd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1131, 'application/route-usd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1132, 'application/rpki-manifest', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1133, 'application/vnd.3gpp.gtpc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1134, 'application/vnd.3gpp.ngap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1135, 'application/vnd.3gpp.pfcp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1136, 'application/vnd.3gpp.s1ap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1137, 'application/vnd.3gpp2.sms', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1138, 'application/vnd.adobe.fxp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1139, 'application/vnd.amiga.ami', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1140, 'application/vnd.artsquare', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1141, 'application/vnd.chess-pgn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1142, 'application/vnd.ctc-posml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1143, 'application/vnd.dece.data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1144, 'application/vnd.dolby.mlp', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1145, 'application/vnd.dtg.local', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1146, 'application/vnd.epson.esf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1147, 'application/vnd.epson.msf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1148, 'application/vnd.epson.ssf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1149, 'application/vnd.fdsn.seed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1150, 'application/vnd.hl7v2+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1151, 'application/vnd.ieee.1905', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1152, 'application/vnd.kde.kivio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1153, 'application/vnd.kde.kword', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1154, 'application/vnd.leap+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1155, 'application/vnd.nokia.ncd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1156, 'application/vnd.pawaafile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1157, 'application/vnd.pg.format', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1158, 'application/vnd.pg.osasli', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1159, 'application/vnd.pvi.ptid1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1160, 'application/vnd.seis+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1161, 'application/vnd.sycle+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1162, 'application/vnd.syft+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1163, 'application/vnd.uiq.theme', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1164, 'application/vnd.visionary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1165, 'application/vnd.wap.wbxml', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1166, 'application/whoispp-query', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1167, 'application/x-font-libgrx', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1168, 'application/x-font-speedo', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1169, 'application/x-ms-shortcut', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1170, 'application/x-msmediaview', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1171, 'application/x-mspublisher', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1172, 'application/x-pki-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1173, 'application/x-wais-source', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1174, 'application/xcap-caps+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1175, 'application/xcap-diff+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1176, 'application/xml-patch+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1177, 'application/yang-data+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1178, 'audio/vnd.nuera.ecelp4800', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1179, 'audio/vnd.nuera.ecelp7470', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1180, 'audio/vnd.nuera.ecelp9600', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1181, 'image/vnd.adobe.photoshop', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1182, 'multipart/x-mixed-replace', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1183, 'text/tab-separated-values', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1184, 'text/vnd.debian.copyright', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1185, 'video/vnd.motorola.videop', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1186, 'video/vnd.nokia.videovoip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1187, 'application/bacnet-xdd+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1188, 'application/cdmi-container', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1189, 'application/dash-patch+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1190, 'application/index.response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1191, 'application/mac-compactpro', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1192, 'application/moss-signature', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1193, 'application/multipart-core', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1194, 'application/news-groupinfo', 'iana', NULL, 'US-ASCII');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1195, 'application/pkix-attr-cert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1196, 'application/provenance+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1197, 'application/vnd.3gpp.5gnas', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1198, 'application/vnd.3gpp2.tcap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1199, 'application/vnd.adobe.xfdf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1200, 'application/vnd.aether.imp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1201, 'application/vnd.afpc.modca', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1202, 'application/vnd.ah-barcode', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1203, 'application/vnd.audiograph', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1204, 'application/vnd.canon-cpdl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1205, 'application/vnd.canon-lips', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1206, 'application/vnd.cinderella', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1207, 'application/vnd.curl.pcurl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1208, 'application/vnd.drive+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1209, 'application/vnd.epson.salt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1210, 'application/vnd.etsi.mheg5', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1211, 'application/vnd.fdsn.mseed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1212, 'application/vnd.filmit.zfc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1213, 'application/vnd.flographit', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1214, 'application/vnd.framemaker', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1215, 'application/vnd.fut-misnet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1216, 'application/vnd.fuzzysheet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1217, 'application/vnd.hl7cda+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1218, 'application/vnd.hyper+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1219, 'application/vnd.ibm.modcap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1220, 'application/vnd.iccprofile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1221, 'application/vnd.insors.igm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1222, 'application/vnd.kde.karbon', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1223, 'application/vnd.kde.kchart', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1224, 'application/vnd.kenameaapp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1225, 'application/vnd.mason+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1226, 'application/vnd.micro+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1227, 'application/vnd.miele+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1228, 'application/vnd.mobius.daf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1229, 'application/vnd.mobius.dis', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1230, 'application/vnd.mobius.mbk', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1231, 'application/vnd.mobius.mqy', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1232, 'application/vnd.mobius.msl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1233, 'application/vnd.mobius.plc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1234, 'application/vnd.mobius.txf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1235, 'application/vnd.ms-outlook', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1236, 'application/vnd.ms-pki.stl', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1237, 'application/vnd.ms-project', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1238, 'application/vnd.music-niff', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1239, 'application/vnd.olpc-sugar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1240, 'application/vnd.patentdive', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1241, 'application/vnd.pmi.widget', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1242, 'application/vnd.sealed.3df', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1243, 'application/vnd.sealed.csf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1244, 'application/vnd.sealed.doc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1245, 'application/vnd.sealed.eml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1246, 'application/vnd.sealed.mht', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1247, 'application/vnd.sealed.net', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1248, 'application/vnd.sealed.ppt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1249, 'application/vnd.sealed.xls', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1250, 'application/vnd.siren+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1251, 'application/vnd.syncml+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1252, 'application/vnd.wv.csp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1253, 'application/vnd.wv.ssp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1254, 'application/vnd.xacml+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1255, 'application/vnd.xmpie.cpkg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1256, 'application/vnd.xmpie.dpkg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1257, 'application/vnd.xmpie.plan', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1258, 'application/vnd.xmpie.ppkg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1259, 'application/vnd.xmpie.xlim', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1260, 'application/vnd.youtube.yt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1261, 'application/wordperfect5.1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1262, 'application/x-futuresplash', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1263, 'application/x-lua-bytecode', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1264, 'application/x-x509-ca-cert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1265, 'application/xcap-error+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1266, 'application/yang-data+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1267, 'application/yang-patch+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1268, 'audio/vnd.nokia.mobile-xmf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1269, 'text/x-handlebars-template', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1270, 'video/vnd.directv.mpeg-tts', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1271, 'video/vnd.iptvforum.ttsavc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1272, 'application/alto-error+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1273, 'application/atomdeleted+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1274, 'application/auth-policy+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1275, 'application/call-completion', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1276, 'application/cdmi-capability', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1277, 'application/coap-group+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1278, 'application/dialog-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1279, 'application/jscalendar+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1280, 'application/json-patch+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1281, 'application/mosskey-request', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1282, 'application/mrb-publish+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1283, 'application/opc-nodeset+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1284, 'application/p2p-overlay+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1285, 'application/pkcs7-signature', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1286, 'application/pkcs8-encrypted', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1287, 'application/remote-printing', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1288, 'application/scvp-cv-request', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1289, 'application/scvp-vp-request', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1290, 'application/senml-etch+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1291, 'application/senml-etch+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1292, 'application/timestamp-query', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1293, 'application/timestamp-reply', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1294, 'application/vnd.ahead.space', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1295, 'application/vnd.android.ota', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1296, 'application/vnd.apple.pages', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1297, 'application/vnd.arastra.swi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1298, 'application/vnd.autopackage', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1299, 'application/vnd.avalon+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1300, 'application/vnd.avistar+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1301, 'application/vnd.bbf.usp.msg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1302, 'application/vnd.cab-jscript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1303, 'application/vnd.cloanto.rp9', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1304, 'application/vnd.commonspace', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1305, 'application/vnd.cosmocaller', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1306, 'application/vnd.ctct.ws+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1307, 'application/vnd.cups-raster', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1308, 'application/vnd.d3m-dataset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1309, 'application/vnd.d3m-problem', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1310, 'application/vnd.ds-keypoint', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1311, 'application/vnd.dvb.service', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1312, 'application/vnd.eudora.data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1313, 'application/vnd.ezpix-album', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1314, 'application/vnd.frogans.fnc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1315, 'application/vnd.frogans.ltf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1316, 'application/vnd.futoin+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1317, 'application/vnd.futoin+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1318, 'application/vnd.geocube+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1319, 'application/vnd.groove-help', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1320, 'application/vnd.heroku+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1321, 'application/vnd.ibm.minipay', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1322, 'application/vnd.kde.kontour', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1323, 'application/vnd.kde.kspread', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1324, 'application/vnd.las.las+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1325, 'application/vnd.lotus-1-2-3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1326, 'application/vnd.lotus-notes', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1327, 'application/vnd.medcalcdata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1328, 'application/vnd.ms-artgalry', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1329, 'application/vnd.ms-htmlhelp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1330, 'application/vnd.ms-opentype', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1331, 'application/vnd.muvee.style', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1332, 'application/vnd.ncd.control', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1333, 'application/vnd.oma.dd2+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1334, 'application/vnd.oma.pal+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1335, 'application/vnd.onepagertat', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1336, 'application/vnd.openeye.oeb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1337, 'application/vnd.osgi.bundle', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1338, 'application/vnd.pocketlearn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1339, 'application/vnd.realvnc.bed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1340, 'application/vnd.sealed.tiff', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1341, 'application/vnd.vectorworks', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1342, 'application/vnd.wordperfect', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1343, 'application/watcherinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1344, 'application/x-7z-compressed', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1345, 'application/x-iso9660-image', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1346, 'application/x-msdos-program', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1347, 'application/xhtml-voice+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1348, 'application/yang-patch+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1349, 'audio/vnd.rhetorex.32kadpcm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1350, 'audio/x-pn-realaudio-plugin', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1351, 'multipart/vnd.bint.med-plus', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1352, 'text/vnd.trolltech.linguist', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1353, 'application/kpml-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1354, 'application/load-control+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1355, 'application/merge-patch+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1356, 'application/mrb-consumer+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1357, 'application/news-checkgroups', 'iana', NULL, 'US-ASCII');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1358, 'application/poc-settings+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1359, 'application/rls-services+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1360, 'application/route-s-tsid+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1361, 'application/rpki-publication', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1362, 'application/samlmetadata+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1363, 'application/scvp-cv-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1364, 'application/scvp-vp-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1365, 'application/set-registration', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1366, 'application/soap+fastinfoset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1367, 'application/tamp-apex-update', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1368, 'application/timestamped-data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1369, 'application/urc-grpsheet+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1370, 'application/urc-ressheet+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1371, 'application/vnd.3gpp.bsf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1372, 'application/vnd.3gpp.sms+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1373, 'application/vnd.amadeus+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1374, 'application/vnd.amazon.ebook', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1375, 'application/vnd.apple.pkpass', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1376, 'application/vnd.artisan+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1377, 'application/vnd.chemdraw+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1378, 'application/vnd.coffeescript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1379, 'application/vnd.contact.cmsg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1380, 'application/vnd.dreamfactory', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1381, 'application/vnd.ecdis-update', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1382, 'application/vnd.ecowin.chart', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1383, 'application/vnd.eszigno3+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1384, 'application/vnd.etsi.aoc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1385, 'application/vnd.etsi.cug+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1386, 'application/vnd.etsi.sci+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1387, 'application/vnd.etsi.tsl+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1388, 'application/vnd.etsi.tsl.der', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1389, 'application/vnd.groove-vcard', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1390, 'application/vnd.jsk.isdn-ngn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1391, 'application/vnd.kde.kformula', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1392, 'application/vnd.kidspiration', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1393, 'application/vnd.las.las+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1394, 'application/vnd.noblenet-web', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1395, 'application/vnd.novadigm.edm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1396, 'application/vnd.novadigm.edx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1397, 'application/vnd.novadigm.ext', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1398, 'application/vnd.oipf.pae.gem', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1399, 'application/vnd.onepagertamp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1400, 'application/vnd.onepagertamx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1401, 'application/vnd.onepagertatp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1402, 'application/vnd.onepagertatx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1403, 'application/vnd.restful+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1404, 'application/vnd.rn-realmedia', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1405, 'application/vnd.spotfire.dxp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1406, 'application/vnd.spotfire.sfs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1407, 'application/vnd.sun.wadl+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1408, 'application/vnd.sun.xml.calc', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1409, 'application/vnd.sun.xml.draw', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1410, 'application/vnd.sun.xml.math', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1411, 'application/vnd.sus-calendar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1412, 'application/vnd.tcpdump.pcap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1413, 'application/vnd.triscape.mxs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1414, 'application/vnd.uplanet.list', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1415, 'application/vnd.veryant.thin', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1416, 'application/vnd.wv.csp+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1417, 'application/vnd.xfdl.webform', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1418, 'application/voucher-cms+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1419, 'application/whoispp-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1420, 'application/x-ace-compressed', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1421, 'application/x-authorware-bin', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1422, 'application/x-authorware-map', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1423, 'application/x-authorware-seg', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1424, 'application/x-cfs-compressed', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1425, 'application/x-debian-package', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1426, 'application/x-dgc-compressed', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1427, 'application/x-font-linux-psf', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1428, 'application/x-gca-compressed', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1429, 'application/x-java-jnlp-file', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1430, 'application/x-lzh-compressed', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1431, 'application/x-ms-application', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1432, 'application/x-rar-compressed', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1433, 'application/x-virtualbox-hdd', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1434, 'application/x-virtualbox-ova', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1435, 'application/x-virtualbox-ovf', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1436, 'application/x-virtualbox-vdi', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1437, 'application/x-virtualbox-vhd', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1438, 'audio/vnd.cmles.radio-events', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1439, 'image/vnd.globalgraphics.pgb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1440, 'text/vnd.familysearch.gedcom', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1441, 'video/vnd.iptvforum.ttsmpeg2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1442, 'video/vnd.radgamettools.bink', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1443, 'application/3gpphalforms+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1444, 'application/alto-costmap+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1445, 'application/ibe-pkg-reply+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1446, 'application/kpml-response+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1447, 'application/mbms-envelope+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1448, 'application/mbms-register+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1449, 'application/mbms-schedule+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1450, 'application/media_control+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1451, 'application/news-transmission', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1452, 'application/oebps-package+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1453, 'application/ppsp-tracker+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1454, 'application/prs.rdf-xml-crypt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1455, 'application/rpki-ghostbusters', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1456, 'application/samlassertion+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1457, 'application/sgml-open-catalog', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1458, 'application/simple-filter+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1459, 'application/spirits-event+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1460, 'application/tamp-status-query', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1461, 'application/vnd.3gpp.gmop+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1462, 'application/vnd.3gpp.ussd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1463, 'application/vnd.adobe.xdp+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1464, 'application/vnd.apple.keynote', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1465, 'application/vnd.apple.mpegurl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1466, 'application/vnd.apple.numbers', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1467, 'application/vnd.balsamiq.bmpr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1468, 'application/vnd.bbf.usp.error', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1469, 'application/vnd.byu.uapi+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1470, 'application/vnd.clonk.c4group', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1471, 'application/vnd.comicbook+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1472, 'application/vnd.comicbook-rar', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1473, 'application/vnd.crick.clicker', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1474, 'application/vnd.cyclonedx+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1475, 'application/vnd.dece.ttml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1476, 'application/vnd.desmume.movie', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1477, 'application/vnd.document+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1478, 'application/vnd.ecowin.series', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1479, 'application/vnd.enphase.envoy', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1480, 'application/vnd.etsi.mcid+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1481, 'application/vnd.etsi.pstn+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1482, 'application/vnd.ezpix-package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1483, 'application/vnd.fluxtime.clip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1484, 'application/vnd.fsc.weblaunch', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1485, 'application/vnd.fujitsu.oasys', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1486, 'application/vnd.fujixerox.ddd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1487, 'application/vnd.geogebra.file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1488, 'application/vnd.geogebra.tool', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1489, 'application/vnd.hcl-bireports', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1490, 'application/vnd.ims.imsccv1p1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1491, 'application/vnd.ims.imsccv1p2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1492, 'application/vnd.ims.imsccv1p3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1493, 'application/vnd.lotus-wordpro', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1494, 'application/vnd.motorola.iprm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1495, 'application/vnd.ms-fontobject', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1496, 'application/vnd.ms-pki.seccat', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1497, 'application/vnd.ms-powerpoint', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1498, 'application/vnd.ncd.reference', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1499, 'application/vnd.nebumind.line', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1500, 'application/vnd.nokia.pcd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1501, 'application/vnd.oma.lwm2m+tlv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1502, 'application/vnd.orange.indata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1503, 'application/vnd.osa.netdeploy', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1504, 'application/vnd.powerbuilder6', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1505, 'application/vnd.powerbuilder7', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1506, 'application/vnd.rainstor.data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1507, 'application/vnd.shopkick+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1508, 'application/vnd.smart.teacher', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1509, 'application/vnd.street-stream', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1510, 'application/vnd.swiftview-ics', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1511, 'application/vnd.syncml.dm+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1512, 'application/vnd.tri.onesource', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1513, 'application/vnd.uplanet.alert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1514, 'application/vnd.ves.encrypted', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1515, 'application/vnd.wmf.bootstrap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1516, 'application/vnd.yamaha.hv-dic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1517, 'application/x-apple-diskimage', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1518, 'application/x-dtbresource+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1519, 'application/x-font-framemaker', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1520, 'application/x-font-sunos-news', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1521, 'application/x-shockwave-flash', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1522, 'application/x-silverlight-app', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1523, 'application/x-virtualbox-vbox', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1524, 'application/x-virtualbox-vmdk', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1525, 'application/x-x509-ca-ra-cert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1526, 'audio/atrac-advanced-lossless', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1527, 'audio/vnd.presonus.multitrack', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1528, 'text/1d-interleaved-parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1529, 'application/geopackage+sqlite3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1530, 'application/im-iscomposing+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1531, 'application/mathml-content+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1532, 'application/resource-lists+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1533, 'application/sparql-results+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1534, 'application/urc-targetdesc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1535, 'application/vnd.3gpp-prose+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1536, 'application/vnd.biopax.rdf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1537, 'application/vnd.cyclonedx+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1538, 'application/vnd.dolby.mobile.1', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1539, 'application/vnd.dolby.mobile.2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1540, 'application/vnd.dtg.local.html', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1541, 'application/vnd.dvb.dvbisl+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1542, 'application/vnd.dvb.ipdcesgpdd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1543, 'application/vnd.ficlab.flb+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1544, 'application/vnd.fujitsu.oasys2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1545, 'application/vnd.fujitsu.oasys3', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1546, 'application/vnd.fujixerox.art4', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1547, 'application/vnd.fujixerox.hbpl', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1548, 'application/vnd.groove-account', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1549, 'application/vnd.kde.kpresenter', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1550, 'application/vnd.lotus-approach', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1551, 'application/vnd.micrografx.flo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1552, 'application/vnd.micrografx.igx', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1553, 'application/vnd.ms-3mfdocument', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1554, 'application/vnd.ms-officetheme', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1555, 'application/vnd.ms-xpsdocument', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1556, 'application/vnd.msa-disk-image', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1557, 'application/vnd.nokia.catalogs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1558, 'application/vnd.oftn.l10n+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1559, 'application/vnd.oma.bcast.ltkm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1560, 'application/vnd.oma.bcast.sgdu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1561, 'application/vnd.oma.bcast.stkm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1562, 'application/vnd.oma.lwm2m+cbor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1563, 'application/vnd.oma.lwm2m+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1564, 'application/vnd.omads-file+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1565, 'application/vnd.osgi.subsystem', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1566, 'application/vnd.pagerduty+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1567, 'application/vnd.powerbuilder75', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1568, 'application/vnd.quarantainenet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1569, 'application/vnd.rig.cryptonote', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1570, 'application/vnd.sigrok.session', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1571, 'application/vnd.smart.notebook', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1572, 'application/vnd.sun.xml.writer', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1573, 'application/vnd.tmobile-livetv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1574, 'application/vnd.uplanet.signal', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1575, 'application/vnd.wap.wmlscriptc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1576, 'application/vnd.wolfram.player', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1577, 'application/vnd.zzazz.deck+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1578, 'application/x-chrome-extension', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1579, 'application/x-font-ghostscript', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1580, 'application/x-mobipocket-ebook', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1581, 'audio/1d-interleaved-parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1582, 'image/vnd.fujixerox.edmics-mmr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1583, 'image/vnd.fujixerox.edmics-rlc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1584, 'image/vnd.valve.source.texture', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1585, 'message/global-delivery-status', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1586, 'video/1d-interleaved-parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1587, 'application/alto-directory+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1588, 'application/conference-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1589, 'application/ibe-key-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1590, 'application/mbms-deregister+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1591, 'application/oauth-authz-req+jwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1592, 'application/patch-ops-error+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1593, 'application/tamp-update-confirm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1594, 'application/trickle-ice-sdpfrag', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1595, 'application/vnd.3gpp.pic-bw-var', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1596, 'application/vnd.businessobjects', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1597, 'application/vnd.collection+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1598, 'application/vnd.cups-postscript', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1599, 'application/vnd.data-vision.rdz', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1600, 'application/vnd.dtg.local.flash', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1601, 'application/vnd.dvb.ipdcroaming', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1602, 'application/vnd.etsi.asic-e+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1603, 'application/vnd.etsi.asic-s+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1604, 'application/vnd.f-secure.mobile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1605, 'application/vnd.fujitsu.oasysgp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1606, 'application/vnd.geogebra.slides', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1607, 'application/vnd.groove-injector', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1608, 'application/vnd.hyper-item+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1609, 'application/vnd.hyperdrive+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1610, 'application/vnd.ibm.afplinedata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1611, 'application/vnd.immervision-ivp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1612, 'application/vnd.immervision-ivu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1613, 'application/vnd.intertrust.nncp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1614, 'application/vnd.iso11783-10+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1615, 'application/vnd.lotus-freelance', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1616, 'application/vnd.lotus-organizer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1617, 'application/vnd.lotus-screencam', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1618, 'application/vnd.marlin.drm.mdcf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1619, 'application/vnd.mozilla.xul+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1620, 'application/vnd.multiad.creator', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1621, 'application/vnd.nearst.inv+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1622, 'application/vnd.noblenet-sealer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1623, 'application/vnd.nokia.conml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1624, 'application/vnd.nokia.pcd+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1625, 'application/vnd.oma-scws-config', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1626, 'application/vnd.oma.cab-pcc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1627, 'application/vnd.omads-email+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1628, 'application/vnd.otps.ct-kip+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1629, 'application/vnd.oxli.countgraph', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1630, 'application/vnd.powerbuilder6-s', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1631, 'application/vnd.powerbuilder7-s', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1632, 'application/vnd.pwg-multiplexed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1633, 'application/vnd.resilient.logic', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1634, 'application/vnd.ruckus.download', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1635, 'application/vnd.shade-save-file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1636, 'application/vnd.shootproof+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1637, 'application/vnd.solent.sdkm+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1638, 'application/vnd.sun.xml.impress', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1639, 'application/vnd.symbian.install', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1640, 'application/vnd.syncml.dm+wbxml', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1641, 'application/vnd.uplanet.cacheop', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1642, 'application/vnd.uplanet.channel', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1643, 'application/vnd.uplanet.listcmd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1644, 'application/vnd.verimatrix.vcas', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1645, 'application/vnd.yamaha.hv-voice', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1646, 'application/x-java-archive-diff', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1647, 'application/x-pkcs7-certreqresp', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1648, 'application/x-x509-next-ca-cert', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1649, 'text/xml-external-parsed-entity', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1650, 'video/vnd.radgamettools.smacker', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1651, 'application/alto-networkmap+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1652, 'application/tamp-sequence-adjust', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1653, 'application/tamp-status-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1654, 'application/urc-uisocketdesc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1655, 'application/vividence.scriptfile', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1656, 'application/vnd.3m.post-it-notes', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1657, 'application/vnd.afpc.afplinedata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1658, 'application/vnd.bbf.usp.msg+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1659, 'application/vnd.bint.med-content', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1660, 'application/vnd.bluetooth.ep.oob', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1661, 'application/vnd.bluetooth.le.oob', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1662, 'application/vnd.cirpack.isdn-ext', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1663, 'application/vnd.datapackage+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1664, 'application/vnd.dece.unspecified', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1665, 'application/vnd.dvb.esgcontainer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1666, 'application/vnd.eprints.data+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1667, 'application/vnd.epson.quickanime', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1668, 'application/vnd.evolv.ecig.theme', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1669, 'application/vnd.exstream-package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1670, 'application/vnd.fujitsu.oasysprs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1671, 'application/vnd.fujixerox.art-ex', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1672, 'application/vnd.genomatix.tuxedo', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1673, 'application/vnd.gentics.grd+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1674, 'application/vnd.google-earth.kmz', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1675, 'application/vnd.hzn-3d-crossword', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1676, 'application/vnd.infotech.project', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1677, 'application/vnd.intercon.formnet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1678, 'application/vnd.kodak-descriptor', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1679, 'application/vnd.macports.portpkg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1680, 'application/vnd.oipf.dae.svg+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1681, 'application/vnd.oipf.spdlist+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1682, 'application/vnd.oma.bcast.sgboot', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1683, 'application/vnd.oma.drm.risd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1684, 'application/vnd.omads-folder+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1685, 'application/vnd.omaloc-supl-init', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1686, 'application/vnd.patientecommsdoc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1687, 'application/vnd.powerbuilder75-s', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1688, 'application/vnd.proteus.magazine', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1689, 'application/vnd.radisys.moml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1690, 'application/vnd.radisys.msml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1691, 'application/vnd.renlearn.rlprint', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1692, 'application/vnd.rn-realmedia-vbr', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1693, 'application/vnd.syncml.dmddf+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1694, 'application/vnd.tableschema+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1695, 'application/vnd.yamaha.hv-script', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1696, 'application/webpush-options+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1697, 'application/x-pkcs7-certificates', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1698, 'audio/vnd.ms-playready.media.pya', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1699, 'image/vnd.airzip.accelerator.azv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1700, 'message/disposition-notification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1701, 'text/vnd.sun.j2me.app-descriptor', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1702, 'video/vnd.ms-playready.media.pyv', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1703, 'application/expect-ct-report+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1704, 'application/mbms-msk-response+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1705, 'application/oblivious-dns-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1706, 'application/pem-certificate-chain', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1707, 'application/simplesymbolcontainer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1708, 'application/tamp-community-update', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1709, 'application/vnd.3gpp.mid-call+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1710, 'application/vnd.3gpp.pic-bw-large', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1711, 'application/vnd.3gpp.pic-bw-small', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1712, 'application/vnd.accpac.simply.aso', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1713, 'application/vnd.accpac.simply.imp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1714, 'application/vnd.adobe.flash.movie', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1715, 'application/vnd.afpc.foca-charset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1716, 'application/vnd.amundsen.maze+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1717, 'application/vnd.apache.arrow.file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1718, 'application/vnd.balsamiq.bmml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1719, 'application/vnd.banana-accounting', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1720, 'application/vnd.blueice.multipass', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1721, 'application/vnd.commerce-battelle', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1722, 'application/vnd.cryptii.pipe+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1723, 'application/vnd.crypto-shade-file', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1724, 'application/vnd.cryptomator.vault', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1725, 'application/vnd.dataresource+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1726, 'application/vnd.dm.delegation+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1727, 'application/vnd.dvb.ipdcesgaccess', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1728, 'application/vnd.ecowin.fileupdate', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1729, 'application/vnd.espass-espass+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1730, 'application/vnd.etsi.iptvsync+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1731, 'application/vnd.etsi.simservs+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1732, 'application/vnd.geometry-explorer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1733, 'application/vnd.gov.sk.e-form+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1734, 'application/vnd.gov.sk.e-form+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1735, 'application/vnd.hhe.lesson-player', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1736, 'application/vnd.ms-cab-compressed', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1737, 'application/vnd.ms-wmdrm.lic-resp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1738, 'application/vnd.neurolanguage.nlu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1739, 'application/vnd.nintendo.snes.rom', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1740, 'application/vnd.nokia.conml+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1741, 'application/vnd.nokia.n-gage.data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1742, 'application/vnd.oma.bcast.imd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1743, 'application/vnd.openblox.game+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1744, 'application/vnd.quark.quarkxpress', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1745, 'application/vnd.stardivision.calc', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1746, 'application/vnd.stardivision.draw', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1747, 'application/vnd.stardivision.math', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1748, 'application/vnd.stepmania.package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1749, 'application/vnd.syncml.dmtnds+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1750, 'application/vnd.ubisoft.webplayer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1751, 'application/vnd.yamaha.smaf-audio', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1752, 'application/x-ns-proxy-autoconfig', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1753, 'application/x-www-form-urlencoded', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1754, 'model/vnd.parasolid.transmit.text', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1755, 'application/3gpdash-qoe-report+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1756, 'application/alto-endpointcost+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1757, 'application/alto-endpointprop+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1758, 'application/fido.trusted-apps+json', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1759, 'application/java-serialized-object', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1760, 'application/mediaservercontrol+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1761, 'application/set-payment-initiation', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1762, 'application/simple-message-summary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1763, 'application/vnd.3gpp.srvcc-ext+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1764, 'application/vnd.afpc.foca-codepage', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1765, 'application/vnd.afpc.modca-cmtable', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1766, 'application/vnd.afpc.modca-formdef', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1767, 'application/vnd.afpc.modca-overlay', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1768, 'application/vnd.amazon.mobi8-ebook', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1769, 'application/vnd.apache.thrift.json', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1770, 'application/vnd.aristanetworks.swi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1771, 'application/vnd.cyan.dean.root+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1772, 'application/vnd.dvb.ipdcesgaccess2', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1773, 'application/vnd.dvb.notif-init+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1774, 'application/vnd.eclipse.ditto+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1775, 'application/vnd.ecowin.filerequest', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1776, 'application/vnd.ericsson.quickcall', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1777, 'application/vnd.evolv.ecig.profile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1778, 'application/vnd.font-fontforge-sfd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1779, 'application/vnd.informix-visionary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1780, 'application/vnd.intertrust.digibox', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1781, 'application/vnd.joost.joda-archive', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1782, 'application/vnd.mapbox-vector-tile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1783, 'application/vnd.maxmind.maxmind-db', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1784, 'application/vnd.mediastation.cdkey', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1785, 'application/vnd.meridian-slingshot', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1786, 'application/vnd.mophun.application', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1787, 'application/vnd.mophun.certificate', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1788, 'application/vnd.motorola.flexsuite', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1789, 'application/vnd.ms-windows.wsd.oob', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1790, 'application/vnd.nacamar.ybrid+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1791, 'application/vnd.nintendo.nitro.rom', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1792, 'application/vnd.noblenet-directory', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1793, 'application/vnd.nokia.landmark+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1794, 'application/vnd.nokia.radio-preset', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1795, 'application/vnd.oipf.dae.xhtml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1796, 'application/vnd.oipf.ueprofile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1797, 'application/vnd.oma.bcast.sgdd+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1798, 'application/vnd.oma.poc.groups+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1799, 'application/vnd.opentimestamps.ots', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1800, 'application/vnd.previewsystems.box', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1801, 'application/vnd.recordare.musicxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1802, 'application/vnd.route66.link66+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1803, 'application/vnd.simtech-mindmapper', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1804, 'application/vnd.snesdev-page-table', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1805, 'application/vnd.syncml.dmddf+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1806, 'application/vnd.uplanet.list-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1807, 'application/vnd.veritone.aion+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1808, 'application/vnd.yamaha.smaf-phrase', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1809, 'application/vnd.yamaha.through-ngn', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1810, 'application/x-install-instructions', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1811, 'application/x-iwork-keynote-sffkey', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1812, 'application/x-iwork-pages-sffpages', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1813, 'image/vnd.sealedmedia.softseal.gif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1814, 'image/vnd.sealedmedia.softseal.jpg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1815, 'text/vnd.esmertec.theme-descriptor', 'iana', NULL, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1816, 'text/vnd.radisys.msml-basic-layout', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1817, 'video/vnd.sealedmedia.softseal.mov', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1818, 'application/alto-costmapfilter+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1819, 'application/mathml-presentation+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1820, 'application/missing-blocks+cbor-seq', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1821, 'application/relax-ng-compact-syntax', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1822, 'application/resource-lists-diff+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1823, 'application/token-introspection+jwt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1824, 'application/vnd.3gpp.mcdata-payload', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1825, 'application/vnd.3gpp.mcptt-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1826, 'application/vnd.3gpp.srvcc-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1827, 'application/vnd.3gpp2.bcmcsinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1828, 'application/vnd.afpc.foca-codedfont', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1829, 'application/vnd.apache.arrow.stream', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1830, 'application/vnd.aplextor.warrp+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1831, 'application/vnd.apple.installer+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1832, 'application/vnd.bekitzur-stech+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1833, 'application/vnd.capasystems-pg+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1834, 'application/vnd.collection.doc+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1835, 'application/vnd.dvb.iptv.alfec-base', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1836, 'application/vnd.ecowin.seriesupdate', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1837, 'application/vnd.etsi.iptvsad-bc+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1838, 'application/vnd.evolv.ecig.settings', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1839, 'application/vnd.fastcopy-disk-image', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1840, 'application/vnd.fujifilm.fb.jfi+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1841, 'application/vnd.fujixerox.docuworks', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1842, 'application/vnd.groove-tool-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1843, 'application/vnd.liberty-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1844, 'application/vnd.ms-color.iccprofile', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1845, 'application/vnd.ms-wmdrm.meter-resp', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1846, 'application/vnd.multiad.creator.cif', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1847, 'application/vnd.nokia.n-gage.ac+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1848, 'application/vnd.nokia.radio-presets', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1849, 'application/vnd.oipf.cspg-hexbinary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1850, 'application/vnd.oma.bcast.sprov+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1851, 'application/vnd.pwg-xhtml-print+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1852, 'application/vnd.stardivision.writer', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1853, 'application/vnd.stepmania.stepchart', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1854, 'application/vnd.syncml.dmtnds+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1855, 'application/vnd.uplanet.alert-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1856, 'application/vnd.wolfram.mathematica', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1857, 'application/vnd.wrq-hp3000-labelled', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1858, 'application/vnd.yamaha.remote-setup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1859, 'application/x-research-info-systems', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1860, 'application/x-web-app-manifest+json', NULL, true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1861, 'audio/vnd.sealedmedia.softseal.mpeg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1862, 'model/vnd.parasolid.transmit.binary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1863, 'model/vnd.valve.source.compiled-map', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1864, 'application/1d-interleaved-parityfec', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1865, 'application/framework-attributes+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1866, 'application/media-policy-dataset+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1867, 'application/tamp-apex-update-confirm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1868, 'application/vnd.3gpp-prose-pc3ch+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1869, 'application/vnd.3gpp.mcdata-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1870, 'application/vnd.adobe.partial-upload', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1871, 'application/vnd.afpc.modca-mediummap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1872, 'application/vnd.americandynamics.acc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1873, 'application/vnd.antix.game-component', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1874, 'application/vnd.apache.thrift.binary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1875, 'application/vnd.chipnuts.karaoke-mmd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1876, 'application/vnd.collection.next+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1877, 'application/vnd.coreos.ignition+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1878, 'application/vnd.ecowin.seriesrequest', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1879, 'application/vnd.etsi.iptvcommand+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1880, 'application/vnd.etsi.iptvprofile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1881, 'application/vnd.etsi.iptvsad-cod+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1882, 'application/vnd.etsi.iptvservice+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1883, 'application/vnd.etsi.timestamp-token', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1884, 'application/vnd.exstream-empower+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1885, 'application/vnd.google-apps.document', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1886, 'application/vnd.google-earth.kml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1887, 'application/vnd.groove-tool-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1888, 'application/vnd.hydrostatix.sof-data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1889, 'application/vnd.ibm.electronic-media', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1890, 'application/vnd.ibm.secure-container', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1891, 'application/vnd.imagemeter.image+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1892, 'application/vnd.infotech.project+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1893, 'application/vnd.iptc.g2.newsitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1894, 'application/vnd.logipipe.circuit+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1895, 'application/vnd.minisoft-hp3000-save', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1896, 'application/vnd.nokia.landmark+wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1897, 'application/vnd.oipf.spdiscovery+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1898, 'application/vnd.oipf.userprofile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1899, 'application/vnd.openblox.game-binary', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1900, 'application/vnd.oracle.resource+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1901, 'application/vnd.sailingtracker.track', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1902, 'application/vnd.stardivision.impress', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1903, 'application/vnd.vividence.scriptfile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1904, 'application/x-redhat-package-manager', 'nginx', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1905, 'application/xcon-conference-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1906, 'video/vnd.iptvforum.1dparityfec-1010', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1907, 'video/vnd.iptvforum.1dparityfec-2005', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1908, 'video/vnd.iptvforum.2dparityfec-1010', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1909, 'video/vnd.iptvforum.2dparityfec-2005', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1910, 'application/emergencycalldata.cap+xml', 'iana', true, 'UTF-8');
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1911, 'application/mbms-reception-report+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1912, 'application/vnd.3gpp.mcptt-signed+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1913, 'application/vnd.3gpp.mcvideo-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1914, 'application/vnd.afpc.cmoca-cmresource', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1915, 'application/vnd.airzip.filesecure.azf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1916, 'application/vnd.airzip.filesecure.azs', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1917, 'application/vnd.apache.thrift.compact', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1918, 'application/vnd.astraea-software.iota', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1919, 'application/vnd.crick.clicker.palette', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1920, 'application/vnd.criticaltools.wbs+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1921, 'application/vnd.cryptomator.encrypted', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1922, 'application/vnd.debian.binary-package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1923, 'application/vnd.dvb.notif-generic+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1924, 'application/vnd.etsi.iptvsad-npvr+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1925, 'application/vnd.eu.kasparian.car+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1926, 'application/vnd.firemonkeys.cloudcell', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1927, 'application/vnd.fujifilm.fb.docuworks', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1928, 'application/vnd.ibm.rights-management', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1929, 'application/vnd.imagemeter.folder+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1930, 'application/vnd.ipunplugged.rcprofile', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1931, 'application/vnd.japannet-registration', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1932, 'application/vnd.japannet-verification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1933, 'application/vnd.jcp.javame.midlet-rms', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1934, 'application/vnd.maxar.archive.3tz+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1935, 'application/vnd.ms-office.activex+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1936, 'application/vnd.ms-wmdrm.lic-chlg-req', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1937, 'application/vnd.nokia.iptv.config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1938, 'application/vnd.oma-scws-http-request', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1939, 'application/vnd.publishare-delta-tree', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1940, 'application/vnd.qualcomm.brew-app-res', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1941, 'application/vnd.quobject-quoxdocument', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1942, 'application/vnd.radisys.msml-conf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1943, 'application/vnd.sun.xml.calc.template', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1944, 'application/vnd.sun.xml.draw.template', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1945, 'application/vnd.sun.xml.writer.global', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1946, 'application/vnd.think-cell.ppttc+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1947, 'application/vnd.tmd.mediaflex.api+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1948, 'application/vnd.uplanet.bearer-choice', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1949, 'application/vnd.uplanet.cacheop-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1950, 'application/vnd.uplanet.channel-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1951, 'application/vnd.uplanet.listcmd-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1952, 'application/vnd.valve.source.material', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1953, 'application/vnd.vidsoft.vidconference', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1954, 'application/vnd.windows.devicepairing', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1955, 'application/x-virtualbox-vbox-extpack', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1956, 'text/vnd.net2phone.commcenter.command', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1957, 'application/alto-networkmapfilter+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1958, 'application/atsc-dynamic-event-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1959, 'application/emergencycalldata.veds+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1960, 'application/mbms-register-response+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1961, 'application/vnd.3gpp.interworking-data', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1962, 'application/vnd.3gpp.mc-signalling-ear', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1963, 'application/vnd.3gpp.mcdata-signalling', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1964, 'application/vnd.afpc.modca-pagesegment', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1965, 'application/vnd.crick.clicker.keyboard', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1966, 'application/vnd.crick.clicker.template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1967, 'application/vnd.crick.clicker.wordbank', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1968, 'application/vnd.denovo.fcselayout-link', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1969, 'application/vnd.dvb.ipdcdftnotifaccess', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1970, 'application/vnd.etsi.iptvdiscovery+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1971, 'application/vnd.etsi.iptvueprofile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1972, 'application/vnd.ims.lis.v2.result+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1973, 'application/vnd.marlin.drm.license+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1974, 'application/vnd.motorola.flexsuite.fis', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1975, 'application/vnd.motorola.flexsuite.kmr', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1976, 'application/vnd.motorola.flexsuite.ttc', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1977, 'application/vnd.motorola.flexsuite.wem', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1978, 'application/vnd.oma-scws-http-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1979, 'application/vnd.oma.cab-user-prefs+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1980, 'application/vnd.oma.scidm.messages+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1981, 'application/vnd.oma.xcap-directory+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1982, 'application/vnd.openstreetmap.data+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1983, 'application/vnd.osgeo.mapguide.package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1984, 'application/vnd.radisys.msml-audit+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1985, 'application/vnd.recordare.musicxml+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1986, 'application/vnd.shana.informed.package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1987, 'application/vnd.syncml.dm.notification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1988, 'application/vnd.syncml.ds.notification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1989, 'application/vnd.yamaha.openscoreformat', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1990, 'application/vnd.yamaha.tunnel-udpencap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1991, 'application/x-iwork-numbers-sffnumbers', NULL, NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1992, 'application/xml-external-parsed-entity', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1993, 'model/vnd.rosette.annotated-data-model', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1994, 'video/vnd.nokia.interleaved-multimedia', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1995, 'application/emergencycalldata.ecall.msd', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1996, 'application/prs.alvestrand.titrax-sheet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1997, 'application/set-registration-initiation', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1998, 'application/vnd.adobe.formscentral.fcdt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (1999, 'application/vnd.android.package-archive', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2000, 'application/vnd.blink-idb-value-wrapper', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2001, 'application/vnd.dvb.notif-container+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2002, 'application/vnd.easykaraoke.cdgdownload', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2003, 'application/vnd.familysearch.gedcom+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2004, 'application/vnd.google-apps.spreadsheet', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2005, 'application/vnd.groove-identity-message', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2006, 'application/vnd.informedcontrol.rms+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2007, 'application/vnd.iptc.g2.catalogitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2008, 'application/vnd.iptc.g2.conceptitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2009, 'application/vnd.iptc.g2.newsmessage+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2010, 'application/vnd.iptc.g2.packageitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2011, 'application/vnd.irepository.package+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2012, 'application/vnd.japannet-payment-wakeup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2013, 'application/vnd.motorola.flexsuite.adsi', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2014, 'application/vnd.ms-wmdrm.meter-chlg-req', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2015, 'application/vnd.ntt-local.content-share', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2016, 'application/vnd.ntt-local.file-transfer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2017, 'application/vnd.ntt-local.sip-ta_remote', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2018, 'application/vnd.oasis.opendocument.text', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2019, 'application/vnd.oma.cab-subs-invite+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2020, 'application/vnd.openofficeorg.extension', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2021, 'application/vnd.radisys.msml-dialog+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2022, 'application/vnd.shana.informed.formdata', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2023, 'application/vnd.sun.xml.writer.template', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2024, 'application/vnd.yellowriver-custom-menu', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2025, 'message/global-disposition-notification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2026, 'application/alto-endpointcostparams+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2027, 'application/alto-endpointpropparams+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2028, 'application/alto-updatestreamparams+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2029, 'application/tamp-sequence-adjust-confirm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2030, 'application/vnd.3gpp.mcptt-ue-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2031, 'application/vnd.afpc.afplinedata-pagedef', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2032, 'application/vnd.citationstyles.style+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2033, 'application/vnd.d2l.coursepackage1p0+zip', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2034, 'application/vnd.dir-bi.plate-dl-nosuffix', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2035, 'application/vnd.dvb.notif-ia-msglist+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2036, 'application/vnd.google-apps.presentation', NULL, false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2037, 'application/vnd.iptc.g2.planningitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2038, 'application/vnd.japannet-jpnstore-wakeup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2039, 'application/vnd.japannet-setstore-wakeup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2040, 'application/vnd.marlin.drm.conftoken+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2041, 'application/vnd.motorola.flexsuite.gotap', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2042, 'application/vnd.ms-printschematicket+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2043, 'application/vnd.ms-windows.devicepairing', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2044, 'application/vnd.nokia.isds-radio-presets', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2045, 'application/vnd.oasis.opendocument.chart', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2046, 'application/vnd.oasis.opendocument.image', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2047, 'application/vnd.oma.cab-address-book+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2048, 'application/vnd.oma.group-usage-list+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2049, 'application/vnd.oma.poc.final-report+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2050, 'application/vnd.sealedmedia.softseal.pdf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2051, 'application/vnd.sun.xml.impress.template', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2052, 'application/alto-updatestreamcontrol+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2053, 'application/emergencycalldata.comment+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2054, 'application/emergencycalldata.control+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2055, 'application/tamp-community-update-confirm', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2056, 'application/vnd.3gpp.mcdata-ue-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2057, 'application/vnd.3lightssoftware.imagescal', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2058, 'application/vnd.ims.lti.v2.toolproxy+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2059, 'application/vnd.iptc.g2.knowledgeitem+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2060, 'application/vnd.ms-windows.nwprinting.oob', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2061, 'application/vnd.ms-windows.printerpairing', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2062, 'application/vnd.oma.bcast.drm-trigger+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2063, 'application/vnd.sealedmedia.softseal.html', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2064, 'application/vnd.tao.intent-module-archive', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2065, 'application/xcon-conference-info-diff+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2066, 'application/sarif-external-properties+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2067, 'application/vnd.3gpp.mcvideo-ue-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2068, 'application/vnd.afpc.modca-objectcontainer', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2069, 'application/vnd.cendio.thinlinc.clientconf', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2070, 'application/vnd.century-systems.tcp_stream', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2071, 'application/vnd.dvb.iptv.alfec-enhancement', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2072, 'application/vnd.emclient.accessrequest+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2073, 'application/vnd.fujixerox.docuworks.binder', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2074, 'application/vnd.handheld-entertainment+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2075, 'application/vnd.innopath.wamp.notification', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2076, 'application/vnd.japannet-directory-service', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2077, 'application/vnd.marlin.drm.actiontoken+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2078, 'application/vnd.ms-playready.initiator+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2079, 'application/vnd.oasis.opendocument.formula', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2080, 'application/vnd.oci.image.manifest.v1+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2081, 'application/vnd.oma.bcast.notification+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2082, 'application/vnd.shana.informed.interchange', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2083, 'application/vnd.stardivision.writer-global', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2084, 'application/mbms-protection-description+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2085, 'application/vnd.3gpp.mcptt-user-profile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2086, 'application/vnd.gov.sk.xmldatacontainer+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2087, 'application/vnd.ms-printing.printticket+xml', 'apache', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2088, 'application/vnd.ntt-local.ogw_remote-access', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2089, 'application/vnd.ntt-local.sip-ta_tcp_stream', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2090, 'application/vnd.oasis.opendocument.database', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2091, 'application/vnd.oasis.opendocument.graphics', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2092, 'application/vnd.oasis.opendocument.text-web', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2093, 'application/vnd.oma.cab-feature-handler+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2094, 'application/vnd.poc.group-advertisement+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2095, 'application/vnd.radisys.msml-audit-conf+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2096, 'application/vnd.radisys.msml-audit-conn+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2097, 'application/vnd.shana.informed.formtemplate', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2098, 'application/vnd.software602.filler.form+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2099, 'application/vnd.uplanet.bearer-choice-wbxml', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2100, 'application/vnd.wolfram.mathematica.package', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2101, 'application/emergencycalldata.deviceinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2102, 'application/vnd.1000minds.decision-model+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2103, 'application/vnd.3gpp.mcdata-user-profile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2104, 'application/vnd.3gpp.mcptt-floor-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2105, 'application/vnd.3gpp.mcptt-location-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2106, 'application/vnd.apothekende.reservation+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2107, 'application/vnd.cluetrust.cartomobile-config', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2108, 'application/vnd.dvb.notif-aggregate-root+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2109, 'application/vnd.fujifilm.fb.docuworks.binder', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2110, 'application/vnd.ims.lti.v2.toolproxy.id+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2111, 'application/vnd.ims.lti.v2.toolsettings+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2112, 'application/vnd.japannet-registration-wakeup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2113, 'application/vnd.japannet-verification-wakeup', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2114, 'application/vnd.nokia.landmarkcollection+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2115, 'application/vnd.nokia.n-gage.symbian.install', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2116, 'application/vnd.oipf.mippvcontrolmessage+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2117, 'application/vnd.piaccess.application-licence', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2118, 'application/vnd.radisys.msml-dialog-base+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2119, 'application/emergencycalldata.serviceinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2120, 'application/mbms-user-service-description+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2121, 'application/vnd.3gpp.mcptt-service-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2122, 'application/vnd.3gpp.mcptt-ue-init-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2123, 'application/vnd.3gpp.mcvideo-user-profile+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2124, 'application/vnd.3gpp.state-and-event-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2125, 'application/vnd.collabio.xodocuments.document', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2126, 'application/vnd.fujixerox.docuworks.container', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2127, 'application/vnd.microsoft.portable-executable', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2128, 'application/vnd.oma.bcast.provisioningtrigger', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2129, 'application/vnd.radisys.msml-audit-dialog+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2130, 'application/vnd.radisys.msml-audit-stream+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2131, 'application/vnd.radisys.msml-dialog-group+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2132, 'application/emergencycalldata.providerinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2133, 'application/vnd.3gpp.mcdata-service-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2134, 'application/vnd.3gpp.mcptt-mbms-usage-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2135, 'application/vnd.3gpp.mcvideo-location-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2136, 'application/vnd.ms-excel.addin.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2137, 'application/vnd.ms-excel.sheet.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2138, 'application/vnd.ms-package.obfuscated-opentype', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2139, 'application/vnd.ms-printdevicecapabilities+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2140, 'application/vnd.oasis.opendocument.spreadsheet', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2141, 'application/vnd.oasis.opendocument.text-master', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2142, 'application/vnd.oipf.contentaccessdownload+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2143, 'application/vnd.radisys.msml-dialog-speech+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2144, 'application/vnd.3gpp.access-transfer-events+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2145, 'application/vnd.3gpp.mcvideo-service-config+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2146, 'application/vnd.fujifilm.fb.docuworks.container', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2147, 'application/vnd.globalplatform.card-content-mgt', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2148, 'application/vnd.mitsubishi.misty-guard.trustweb', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2149, 'application/vnd.oasis.opendocument.presentation', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2150, 'application/vnd.oipf.contentaccessstreaming+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2151, 'application/vnd.oma.bcast.smartcard-trigger+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2152, 'application/vnd.software602.filler.form-xml-zip', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2153, 'application/emergencycalldata.subscriberinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2154, 'application/vnd.3gpp.mcvideo-mbms-usage-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2155, 'application/vnd.cluetrust.cartomobile-config-pkg', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2156, 'application/vnd.collabio.xodocuments.spreadsheet', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2157, 'application/vnd.ms-word.document.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2158, 'application/vnd.ms-word.template.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2159, 'application/vnd.oasis.opendocument.text-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2160, 'application/vnd.3gpp.mcvideo-affiliation-info+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2161, 'application/vnd.collabio.xodocuments.presentation', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2162, 'application/vnd.microsoft.windows.thumbnail-cache', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2163, 'application/vnd.ms-excel.template.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2164, 'application/vnd.oasis.opendocument.chart-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2165, 'application/vnd.oasis.opendocument.image-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2166, 'application/vnd.oma.bcast.simple-symbol-container', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2167, 'application/vnd.oma.poc.invocation-descriptor+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2168, 'application/vnd.radisys.msml-dialog-transform+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2169, 'application/vnd.yamaha.openscoreformat.osfpvg+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2170, 'application/vnd.3gpp-v2x-local-service-information', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2171, 'application/vnd.3gpp.mcptt-affiliation-command+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2172, 'application/vnd.doremir.scorecloud-binary-document', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2173, 'application/vnd.llamagraphics.life-balance.desktop', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2174, 'application/vnd.radisys.msml-dialog-fax-detect+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2175, 'application/vnd.3gpp.mcdata-affiliation-command+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2176, 'application/vnd.anser-web-funds-transfer-initiation', 'apache', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2177, 'application/vnd.ims.lti.v2.toolconsumerprofile+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2178, 'application/vnd.ims.lti.v2.toolsettings.simple+json', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2179, 'application/vnd.ms-powerpoint.addin.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2180, 'application/vnd.ms-powerpoint.slide.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2181, 'application/vnd.oasis.opendocument.formula-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2182, 'application/vnd.3gpp.mcvideo-affiliation-command+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2183, 'application/vnd.oasis.opendocument.graphics-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2184, 'application/vnd.oma.poc.detailed-progress-report+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2185, 'application/vnd.radisys.msml-dialog-fax-sendrecv+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2186, 'application/mbms-associated-procedure-description+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2187, 'application/vnd.3gpp.mcvideo-transmission-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2188, 'application/vnd.dvb.notif-ia-registration-request+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2189, 'application/vnd.ms-excel.sheet.binary.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2190, 'application/vnd.oma.poc.optimized-progress-report+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2191, 'application/vnd.anser-web-certificate-issue-initiation', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2192, 'application/vnd.collabio.xodocuments.document-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2193, 'application/vnd.dvb.notif-ia-registration-response+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2194, 'application/vnd.ms-powerpoint.template.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2195, 'application/vnd.llamagraphics.life-balance.exchange+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2196, 'application/vnd.ms-powerpoint.slideshow.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2197, 'application/vnd.oasis.opendocument.spreadsheet-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2198, 'application/vnd.openxmlformats-officedocument.theme+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2199, 'application/vnd.etsi.overload-control-policy-dataset+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2200, 'application/vnd.globalplatform.card-content-mgt-response', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2201, 'application/vnd.oasis.opendocument.presentation-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2202, 'application/vnd.openxmlformats-officedocument.vmldrawing', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2203, 'application/vnd.openxmlformats-package.relationships+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2204, 'application/vnd.collabio.xodocuments.spreadsheet-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2205, 'application/vnd.openxmlformats-officedocument.drawing+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2206, 'application/vnd.collabio.xodocuments.presentation-template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2207, 'application/vnd.ms-powerpoint.presentation.macroenabled.12', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2208, 'application/vnd.openxmlformats-package.core-properties+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2209, 'application/vnd.adobe.air-application-installer-package+zip', 'apache', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2210, 'application/vnd.oma.bcast.associated-procedure-parameter+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2211, 'application/vnd.openxmlformats-officedocument.themeoverride+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2212, 'application/vnd.openxmlformats-officedocument.drawingml.chart+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2213, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2214, 'application/vnd.openxmlformats-officedocument.presentationml.slide', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2215, 'application/vnd.openxmlformats-officedocument.custom-properties+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2216, 'application/vnd.openxmlformats-officedocument.spreadsheetml.template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2217, 'application/vnd.openxmlformats-officedocument.customxmlproperties+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2218, 'application/vnd.openxmlformats-officedocument.extended-properties+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2219, 'application/vnd.openxmlformats-officedocument.presentationml.tags+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2220, 'application/vnd.openxmlformats-officedocument.presentationml.template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2221, 'application/vnd.openxmlformats-officedocument.spreadsheetml.table+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2222, 'application/vnd.openxmlformats-officedocument.presentationml.slide+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2223, 'application/vnd.openxmlformats-officedocument.presentationml.slideshow', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2224, 'application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2225, 'application/vnd.openxmlformats-officedocument.drawingml.chartshapes+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2226, 'application/vnd.openxmlformats-officedocument.drawingml.diagramdata+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2227, 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2228, 'application/vnd.openxmlformats-officedocument.wordprocessingml.template', 'iana', NULL, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2229, 'application/vnd.openxmlformats-officedocument.drawingml.diagramstyle+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2230, 'application/vnd.openxmlformats-officedocument.spreadsheetml.comments+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2231, 'application/vnd.openxmlformats-officedocument.drawingml.diagramcolors+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2232, 'application/vnd.openxmlformats-officedocument.drawingml.diagramlayout+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2233, 'application/vnd.openxmlformats-officedocument.presentationml.comments+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2234, 'application/vnd.openxmlformats-officedocument.presentationml.presentation', 'iana', false, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2235, 'application/vnd.openxmlformats-officedocument.spreadsheetml.calcchain+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2236, 'application/vnd.openxmlformats-officedocument.spreadsheetml.usernames+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2237, 'application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2238, 'application/vnd.openxmlformats-officedocument.wordprocessingml.footer+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2239, 'application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2240, 'application/vnd.openxmlformats-package.digital-signature-xmlsignature+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2241, 'application/vnd.openxmlformats-officedocument.presentationml.presprops+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2242, 'application/vnd.openxmlformats-officedocument.presentationml.viewprops+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2243, 'application/vnd.openxmlformats-officedocument.spreadsheetml.chartsheet+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2244, 'application/vnd.openxmlformats-officedocument.spreadsheetml.pivottable+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2245, 'application/vnd.openxmlformats-officedocument.spreadsheetml.querytable+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2246, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2247, 'application/vnd.openxmlformats-officedocument.presentationml.notesslide+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2248, 'application/vnd.openxmlformats-officedocument.spreadsheetml.connections+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2249, 'application/vnd.openxmlformats-officedocument.spreadsheetml.dialogsheet+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2250, 'application/vnd.openxmlformats-officedocument.spreadsheetml.revisionlog+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2251, 'application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2252, 'application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2253, 'application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2254, 'application/vnd.openxmlformats-officedocument.presentationml.notesmaster+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2255, 'application/vnd.openxmlformats-officedocument.presentationml.slidelayout+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2256, 'application/vnd.openxmlformats-officedocument.presentationml.slidemaster+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2257, 'application/vnd.openxmlformats-officedocument.presentationml.tablestyles+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2258, 'application/vnd.openxmlformats-officedocument.spreadsheetml.externallink+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2259, 'application/vnd.openxmlformats-officedocument.wordprocessingml.fonttable+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2260, 'application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2261, 'application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2262, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sharedstrings+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2263, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheetmetadata+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2264, 'application/vnd.openxmlformats-officedocument.spreadsheetml.template.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2265, 'application/vnd.openxmlformats-officedocument.presentationml.handoutmaster+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2266, 'application/vnd.openxmlformats-officedocument.presentationml.template.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2267, 'application/vnd.openxmlformats-officedocument.wordprocessingml.websettings+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2268, 'application/vnd.openxmlformats-officedocument.presentationml.commentauthors+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2269, 'application/vnd.openxmlformats-officedocument.presentationml.slideshow.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2270, 'application/vnd.openxmlformats-officedocument.spreadsheetml.revisionheaders+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2271, 'application/vnd.openxmlformats-officedocument.presentationml.slideupdateinfo+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2272, 'application/vnd.openxmlformats-officedocument.spreadsheetml.tablesinglecells+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2273, 'application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2274, 'application/vnd.openxmlformats-officedocument.wordprocessingml.template.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2275, 'application/vnd.openxmlformats-officedocument.spreadsheetml.pivotcacherecords+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2276, 'application/vnd.openxmlformats-officedocument.presentationml.presentation.main+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2277, 'application/vnd.openxmlformats-officedocument.spreadsheetml.pivotcachedefinition+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2278, 'application/vnd.openxmlformats-officedocument.spreadsheetml.volatiledependencies+xml', 'iana', true, NULL);
INSERT INTO omni_mimetypes.mime_types OVERRIDING SYSTEM VALUE VALUES (2279, 'application/vnd.openxmlformats-officedocument.wordprocessingml.document.glossary+xml', 'iana', true, NULL);


--
-- Data for Name: mime_types_file_extensions; Type: TABLE DATA; Schema: omni_mimetypes; Owner: -
--

INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1, 1);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (5, 2);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (6, 3);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (7, 4);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (10, 5);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (11, 6);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (13, 7);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (14, 8);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (16, 9);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (18, 10);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 11);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 12);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 13);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 14);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 15);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 16);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (19, 17);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (20, 18);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (25, 19);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (34, 20);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (35, 21);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (35, 22);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (37, 23);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (37, 24);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (37, 25);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (37, 26);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (40, 27);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (43, 28);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (45, 29);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (46, 30);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (47, 31);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (48, 32);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (49, 33);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (50, 34);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (51, 35);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (52, 36);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (52, 37);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (53, 38);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (54, 39);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (55, 40);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (55, 41);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (56, 42);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (57, 43);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (58, 44);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (59, 45);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (60, 46);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (61, 47);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (62, 48);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (63, 49);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (65, 50);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (66, 51);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (67, 52);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (69, 53);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (69, 54);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (69, 55);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (70, 56);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (71, 57);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (72, 58);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (72, 59);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (73, 60);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (74, 61);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (74, 62);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (75, 63);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (77, 64);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (77, 65);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (79, 67);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (80, 68);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (80, 69);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (81, 70);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (81, 71);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (81, 72);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (83, 73);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (90, 74);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (106, 75);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (106, 76);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (106, 77);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (106, 78);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (107, 79);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (107, 80);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (107, 81);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (107, 83);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (107, 84);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (112, 85);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (119, 87);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (120, 88);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (121, 89);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (122, 90);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (123, 91);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (124, 92);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (125, 93);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (126, 94);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (127, 95);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (128, 96);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (129, 97);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (130, 98);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (130, 99);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (130, 100);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (131, 101);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (132, 102);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (133, 103);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (134, 104);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (135, 105);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (136, 106);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (137, 107);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (138, 108);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (138, 109);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (139, 110);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (140, 111);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (140, 112);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (141, 113);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (141, 114);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (141, 115);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (143, 116);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (143, 117);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 118);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 119);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 120);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 121);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 122);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 123);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 124);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (145, 125);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 126);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 127);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 128);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 129);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 130);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (146, 131);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (147, 132);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (149, 133);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (149, 134);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (150, 135);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (151, 136);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (152, 137);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (153, 138);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (154, 139);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (157, 141);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (158, 142);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (159, 143);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (161, 144);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (165, 145);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (166, 146);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (166, 147);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (166, 148);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (166, 149);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (166, 150);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (168, 151);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (170, 152);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (171, 153);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (171, 154);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (183, 155);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (184, 156);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (188, 159);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (189, 160);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (191, 161);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (192, 162);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (193, 163);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (194, 164);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (195, 165);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (196, 166);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (200, 167);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (202, 168);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (202, 169);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (203, 170);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (205, 171);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (206, 172);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (207, 173);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (208, 174);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (211, 175);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (212, 176);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (213, 177);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (214, 178);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (215, 179);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (216, 180);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (229, 181);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (229, 182);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (229, 183);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (230, 184);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (232, 185);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (233, 186);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (233, 187);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (241, 188);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (244, 189);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (266, 190);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (267, 191);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (268, 192);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (269, 193);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (270, 194);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (270, 195);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (271, 196);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (272, 197);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (273, 198);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (274, 199);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (275, 200);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (278, 201);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (279, 202);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (280, 203);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (281, 204);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (282, 205);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (283, 206);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (283, 207);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (284, 208);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (284, 209);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (289, 210);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (289, 211);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (290, 212);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (291, 213);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (291, 214);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (291, 215);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (292, 216);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (295, 217);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (295, 218);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (296, 219);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (301, 220);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (302, 221);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (312, 222);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (313, 223);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (314, 224);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (315, 225);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (316, 226);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (318, 227);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (319, 228);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (320, 229);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (320, 230);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (321, 231);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (323, 233);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (324, 234);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (324, 235);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (326, 236);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (327, 237);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (328, 238);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (328, 239);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (333, 241);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (333, 242);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (333, 243);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (333, 244);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (340, 245);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (341, 246);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (341, 247);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (342, 248);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (343, 249);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (344, 250);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (345, 251);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (359, 252);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (360, 253);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (365, 254);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (365, 255);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (366, 256);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (368, 257);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (370, 258);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (372, 259);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (377, 261);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (380, 263);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (380, 264);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (380, 265);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (381, 266);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (388, 267);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (389, 268);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (390, 269);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (391, 270);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (392, 271);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (393, 272);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (394, 273);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (397, 274);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (405, 275);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (406, 276);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (407, 277);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (412, 278);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (412, 279);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (415, 280);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (417, 281);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (424, 282);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (431, 283);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (431, 284);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (432, 285);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (433, 286);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (435, 287);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (435, 288);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (436, 289);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (437, 290);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (442, 291);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (443, 292);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (444, 293);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (446, 294);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (448, 295);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (449, 296);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (450, 297);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (454, 298);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (456, 299);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (457, 300);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (459, 301);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (460, 302);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (460, 303);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (460, 304);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (460, 305);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (460, 306);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (464, 307);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (464, 308);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (470, 309);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (471, 310);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (472, 311);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (473, 312);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (473, 313);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (473, 314);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (478, 315);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (480, 316);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (481, 317);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (483, 318);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (484, 319);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (484, 320);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (488, 321);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (489, 322);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (489, 323);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (489, 324);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (489, 325);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (489, 326);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (490, 327);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (492, 328);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (493, 329);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (494, 330);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (495, 331);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (496, 332);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (497, 333);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (498, 334);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (499, 335);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (500, 336);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (500, 337);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (501, 338);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (505, 339);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (506, 340);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (507, 341);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (508, 342);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (509, 343);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (510, 344);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (511, 345);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (513, 346);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (513, 347);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (514, 348);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (515, 349);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (516, 350);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (518, 351);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (518, 352);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (519, 353);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (519, 354);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (520, 355);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (520, 356);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (521, 357);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (521, 358);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (522, 359);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (526, 360);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (526, 361);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (528, 362);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (531, 363);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (533, 364);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (534, 365);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (536, 367);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (537, 368);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (538, 369);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (539, 370);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (540, 371);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (542, 372);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (542, 373);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (543, 374);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (544, 375);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (546, 377);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (547, 378);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (547, 379);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (547, 380);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (547, 381);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (558, 382);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (560, 383);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (561, 384);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (562, 385);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (564, 386);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (566, 387);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (568, 388);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (569, 389);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (571, 390);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (574, 391);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (575, 392);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (575, 393);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (576, 394);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (582, 395);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (583, 396);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (585, 397);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (586, 398);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (587, 399);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (588, 400);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (589, 401);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (590, 402);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (592, 403);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (595, 404);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (595, 405);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (595, 406);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (595, 407);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (597, 408);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (598, 409);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (599, 410);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (601, 411);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (601, 412);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (603, 413);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (604, 414);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (607, 415);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (608, 416);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (610, 417);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (610, 418);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (611, 419);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (612, 420);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (614, 421);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (615, 422);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (618, 423);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (619, 424);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (621, 425);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (623, 426);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (624, 427);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (626, 428);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (630, 429);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (632, 430);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (633, 431);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (635, 432);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (636, 433);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (636, 434);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (637, 435);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (638, 436);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (638, 437);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (639, 438);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (639, 439);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (640, 440);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (641, 441);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (642, 442);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (643, 443);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (644, 444);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (644, 445);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (645, 446);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (647, 447);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (648, 448);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (649, 449);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (660, 450);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (661, 451);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (662, 452);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (663, 453);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (665, 454);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (668, 455);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (668, 456);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (673, 457);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (674, 458);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (675, 459);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (681, 460);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (685, 461);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (691, 462);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (692, 463);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (694, 464);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (695, 465);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (697, 466);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (699, 467);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (702, 468);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (707, 469);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (708, 470);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (709, 471);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (710, 472);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (713, 473);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (715, 474);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (716, 475);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (719, 476);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (720, 477);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (721, 478);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (721, 479);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (723, 480);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (724, 481);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (725, 482);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (726, 483);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (727, 484);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (731, 485);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (732, 486);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (733, 487);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (734, 488);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (734, 489);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (734, 490);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (734, 491);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (736, 492);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (737, 493);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (738, 494);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (740, 495);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (740, 496);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (741, 498);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (741, 499);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (744, 500);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (745, 501);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (746, 502);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (747, 503);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (748, 504);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (748, 505);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (749, 506);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (750, 507);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (751, 508);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (752, 509);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (753, 510);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (754, 511);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (755, 512);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (756, 513);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (756, 514);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (757, 515);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (757, 516);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (758, 517);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (759, 518);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (760, 519);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (761, 520);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (763, 522);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (764, 523);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (766, 524);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (766, 525);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (771, 526);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (778, 529);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (778, 530);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (783, 531);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (797, 532);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (797, 533);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (801, 534);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (803, 535);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (805, 536);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (806, 537);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (811, 538);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (822, 539);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (822, 540);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (824, 541);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (827, 542);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (828, 543);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (828, 544);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (828, 545);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (828, 546);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (830, 547);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (831, 548);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (833, 549);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (834, 550);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (835, 551);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (836, 552);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (837, 553);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (838, 554);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (839, 555);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (839, 556);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (841, 557);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (841, 558);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (842, 559);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (843, 560);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (845, 561);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (847, 562);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (853, 564);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (854, 565);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (855, 566);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (855, 567);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (858, 568);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (861, 569);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (864, 570);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (864, 571);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (865, 572);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (866, 573);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (866, 574);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (868, 575);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (869, 576);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (870, 577);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (870, 578);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (871, 579);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (871, 580);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (871, 581);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (876, 582);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (880, 583);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (885, 584);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (885, 585);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (887, 586);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (888, 587);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (889, 588);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (892, 589);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (893, 590);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (894, 591);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (896, 592);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (897, 593);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (898, 594);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 595);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 596);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 597);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 598);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 599);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 600);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 601);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 602);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (900, 603);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (901, 604);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (903, 605);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (904, 606);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (905, 607);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (906, 608);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (907, 609);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (908, 610);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (909, 611);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (910, 612);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 613);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 614);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 615);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 616);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 617);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 618);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 619);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (911, 620);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (913, 621);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (915, 622);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (915, 623);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (915, 624);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (915, 625);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (917, 627);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (919, 628);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (921, 629);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (926, 630);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (927, 631);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (928, 632);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (929, 633);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (931, 634);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (935, 635);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (937, 636);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (939, 637);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (940, 638);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (940, 639);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (940, 640);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (941, 641);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (942, 642);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (947, 643);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (957, 644);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (957, 645);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (958, 646);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (959, 647);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (961, 648);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (964, 649);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (965, 650);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (966, 651);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (967, 652);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (969, 653);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (970, 654);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (971, 655);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (972, 656);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (972, 657);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (976, 658);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (978, 659);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (982, 660);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (987, 661);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (997, 662);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (998, 663);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1000, 665);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1001, 666);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1002, 667);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1003, 668);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1007, 669);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1008, 670);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1009, 671);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1017, 672);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1020, 673);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1021, 674);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1022, 675);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1029, 676);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1032, 677);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1032, 678);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1032, 679);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1036, 680);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1038, 681);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 682);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 683);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 684);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 685);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 686);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 687);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 688);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 689);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 690);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 691);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 692);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 693);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 694);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 695);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 696);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 697);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 698);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 699);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 700);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 701);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 702);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1041, 703);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1042, 704);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1048, 705);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1051, 706);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1053, 707);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1055, 708);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1057, 709);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1058, 710);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1058, 711);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1062, 712);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1064, 713);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1066, 714);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1067, 715);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1068, 716);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1069, 717);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1070, 718);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 719);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 720);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 721);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 722);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 723);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1071, 724);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1072, 725);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1072, 726);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1072, 727);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1072, 728);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1073, 729);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1081, 730);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1082, 731);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1085, 732);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1086, 733);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1087, 734);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1088, 735);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1089, 736);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1090, 737);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1091, 738);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1092, 739);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1093, 740);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1093, 741);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1093, 742);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1093, 743);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1095, 744);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1097, 745);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1098, 748);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1098, 749);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1099, 754);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1100, 755);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1101, 756);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1102, 757);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1103, 758);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1105, 760);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1115, 761);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1117, 762);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1120, 763);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1121, 764);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1122, 765);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1127, 766);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1128, 768);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1130, 769);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1131, 770);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1132, 771);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1138, 772);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1138, 773);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1139, 774);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1142, 775);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1143, 776);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1143, 777);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1143, 778);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1143, 779);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1144, 780);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1146, 781);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1147, 782);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1148, 783);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1149, 784);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1149, 785);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1152, 786);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1153, 787);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1153, 788);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1156, 789);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1157, 790);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1158, 791);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1159, 792);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1163, 793);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1164, 794);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1165, 795);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1169, 796);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1170, 797);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1170, 798);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1170, 799);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1171, 800);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1173, 801);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1174, 802);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1175, 803);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1178, 804);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1179, 805);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1180, 806);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1181, 807);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1183, 808);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1188, 809);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1189, 810);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1191, 811);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1195, 812);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1196, 813);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1198, 814);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1199, 815);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1203, 816);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1206, 817);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1207, 818);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1209, 819);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1211, 820);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1213, 821);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1214, 822);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1214, 823);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1214, 824);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1214, 825);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1216, 826);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1219, 827);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1219, 828);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1219, 829);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1220, 830);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1220, 831);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1221, 832);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1222, 833);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1223, 834);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1224, 835);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1228, 836);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1229, 837);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1230, 838);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1231, 839);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1232, 840);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1233, 841);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1234, 842);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1235, 843);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1237, 846);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1239, 847);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1241, 848);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1251, 849);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1262, 850);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1263, 851);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1264, 852);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1264, 853);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1264, 854);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1269, 855);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1273, 856);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1276, 857);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1284, 858);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1285, 859);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1288, 860);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1289, 861);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1294, 862);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1296, 863);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1303, 864);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1304, 865);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1305, 866);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1310, 867);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1311, 868);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1313, 869);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1314, 870);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1315, 871);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1319, 872);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1321, 873);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1322, 874);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1323, 875);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1324, 876);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1325, 877);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1326, 878);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1327, 879);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1328, 880);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1329, 881);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1331, 882);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1333, 883);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1338, 884);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1339, 885);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1342, 886);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1343, 887);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1344, 888);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1350, 891);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1359, 892);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1360, 893);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1363, 894);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1364, 895);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1368, 896);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1370, 897);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1374, 898);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1375, 899);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1377, 900);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1379, 901);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1380, 902);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1382, 903);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1383, 904);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1383, 905);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1389, 906);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1391, 907);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1392, 908);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1394, 909);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1395, 910);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1396, 911);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1397, 912);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1404, 913);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1405, 914);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1406, 915);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1407, 916);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1408, 917);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1409, 918);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1410, 919);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1411, 920);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1411, 921);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1412, 922);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1412, 923);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1412, 924);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1413, 925);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1420, 926);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1421, 927);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1421, 928);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1421, 929);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1421, 930);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1422, 931);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1423, 932);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1424, 933);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1425, 935);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1426, 936);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1427, 937);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1428, 938);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1429, 939);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1430, 940);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1430, 941);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1431, 942);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1433, 944);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1434, 945);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1435, 946);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1436, 947);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1437, 948);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1440, 949);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1452, 950);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1455, 951);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1463, 952);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1464, 953);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1465, 954);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1466, 955);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1470, 956);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1470, 957);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1470, 958);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1470, 959);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1470, 960);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1473, 961);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1475, 962);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1475, 963);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1482, 964);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1483, 965);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1484, 966);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1485, 967);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1486, 968);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1487, 969);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1488, 970);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1493, 971);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1495, 972);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1496, 973);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1497, 974);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1497, 975);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1497, 976);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1504, 977);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1508, 978);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1511, 979);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1516, 980);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1518, 982);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1521, 983);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1522, 984);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1523, 985);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1524, 986);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1532, 987);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1533, 988);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1534, 989);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1544, 990);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1545, 991);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1548, 992);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1549, 993);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1549, 994);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1550, 995);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1551, 996);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1552, 997);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1554, 998);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1555, 999);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1565, 1000);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1569, 1001);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1572, 1002);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1573, 1003);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1575, 1004);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1576, 1005);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1577, 1006);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1578, 1007);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1579, 1008);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1580, 1010);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1582, 1011);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1583, 1012);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1584, 1013);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1585, 1014);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1592, 1015);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1595, 1016);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1596, 1017);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1599, 1018);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1605, 1019);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1607, 1020);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1611, 1021);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1612, 1022);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1615, 1023);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1617, 1025);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1619, 1026);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1622, 1027);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1637, 1028);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1637, 1029);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1638, 1030);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1639, 1031);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1639, 1032);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1640, 1033);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1645, 1034);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1646, 1035);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1647, 1036);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1656, 1037);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1664, 1038);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1664, 1039);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1667, 1040);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1670, 1041);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1672, 1042);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1674, 1043);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1677, 1044);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1677, 1045);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1678, 1046);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1679, 1047);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1688, 1048);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1692, 1049);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1693, 1050);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1695, 1051);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1697, 1052);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1697, 1053);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1698, 1054);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1699, 1055);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1700, 1056);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1701, 1057);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1702, 1058);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1710, 1059);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1711, 1060);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1712, 1061);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1713, 1062);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1718, 1063);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1720, 1064);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1732, 1065);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1732, 1066);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1735, 1067);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1736, 1068);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1738, 1069);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1741, 1070);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1743, 1071);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1072);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1073);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1074);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1075);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1076);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1744, 1077);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1745, 1078);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1746, 1079);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1747, 1080);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1748, 1081);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1751, 1082);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1752, 1083);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1754, 1084);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1759, 1085);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1760, 1086);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1761, 1087);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1770, 1088);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1781, 1089);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1782, 1090);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1784, 1091);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1786, 1092);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1787, 1093);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1792, 1094);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1794, 1095);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1800, 1096);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1801, 1097);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1802, 1098);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1803, 1099);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1803, 1100);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1808, 1101);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1810, 1102);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1821, 1105);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1822, 1106);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1831, 1107);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1841, 1108);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1842, 1109);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1848, 1111);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1852, 1112);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1852, 1113);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1853, 1114);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1859, 1115);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1860, 1116);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1862, 1117);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1863, 1118);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1866, 1119);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1872, 1120);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1873, 1121);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1875, 1122);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1885, 1123);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1886, 1124);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1887, 1125);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1888, 1126);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1890, 1127);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1901, 1128);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1902, 1129);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1904, 1130);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1915, 1131);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1916, 1132);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1918, 1133);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1919, 1134);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1920, 1135);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1928, 1136);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1930, 1137);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1933, 1138);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1939, 1139);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1943, 1140);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1944, 1141);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1945, 1142);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1955, 1143);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1965, 1144);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1966, 1145);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1967, 1146);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1968, 1147);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1982, 1148);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1983, 1149);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1985, 1150);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1986, 1151);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1989, 1152);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1997, 1154);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1998, 1155);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (1999, 1156);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2004, 1157);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2005, 1158);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2011, 1159);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2018, 1160);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2020, 1161);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2022, 1162);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2023, 1163);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2024, 1164);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2025, 1165);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2032, 1166);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2036, 1167);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2045, 1168);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2046, 1169);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2051, 1170);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2064, 1171);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2073, 1172);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2074, 1173);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2079, 1174);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2082, 1175);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2083, 1176);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2090, 1177);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2091, 1178);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2092, 1179);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2097, 1180);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2098, 1181);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2102, 1182);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2107, 1183);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2115, 1184);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2136, 1185);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2137, 1186);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2140, 1187);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2141, 1188);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2149, 1189);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2155, 1190);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2157, 1191);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2158, 1192);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2159, 1193);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2163, 1194);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2164, 1195);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2165, 1196);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2169, 1197);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2173, 1198);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2176, 1199);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2179, 1200);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2180, 1201);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2181, 1202);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2183, 1203);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2189, 1204);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2191, 1205);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2194, 1206);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2195, 1207);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2196, 1208);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2197, 1209);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2201, 1210);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2207, 1211);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2209, 1212);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2213, 1213);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2214, 1214);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2216, 1215);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2220, 1216);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2223, 1217);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2227, 1218);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2228, 1219);
INSERT INTO omni_mimetypes.mime_types_file_extensions VALUES (2234, 1220);


--
-- Name: file_extensions_id_seq; Type: SEQUENCE SET; Schema: omni_mimetypes; Owner: -
--

SELECT pg_catalog.setval('omni_mimetypes.file_extensions_id_seq', 1220, true);


--
-- Name: mime_types_id_seq; Type: SEQUENCE SET; Schema: omni_mimetypes; Owner: -
--

SELECT pg_catalog.setval('omni_mimetypes.mime_types_id_seq', 2279, true);;
